@@ -1,5 +1,5 @@
 /*
- *Copyright (C) 2004-2006 Qball Cow <Qball@qballcow.nl>
+ *Copyright (C) 2004-2007 Qball Cow <qball@sarine.nl>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,38 +26,50 @@
 #include <sys/stat.h>
 #endif
 #include "config1.h"
-
+#include <libmpd/debug_printf.h>
 typedef enum _ConfigNodeType  {
 	TYPE_CATEGORY,
 	TYPE_ITEM,
 	TYPE_ITEM_MULTIPLE
 } ConfigNodeType;
-
+/**
+ * A config node
+ * 24byte large
+ */
 typedef struct _config_node {
 	struct _config_node *next;
 	struct _config_node *prev;
 	struct _config_node *parent;
 	gchar *name;
 	ConfigNodeType type;
-	/* union this? */
-	struct _config_node *children; /* TYPE_CATEGORY */
-	gchar *value;/* TYPE_ITEM* */
-
+	/* Save some extra memory by using a union
+	 *  It is actually effective because we build a resonable large tree using this
+	 */
+	union{
+		struct _config_node *children; 	/* TYPE_CATEGORY */
+		gchar *value;			/* TYPE_ITEM* */
+	};
 } config_node;
 
 
 typedef struct _config_obj
 {
 	gchar *url;
+	GMutex *lock;
 	config_node *root;
+	int total_size;
 } _config_obj;
 
-void cfg_remove_node(config_obj *cfg, config_node *node);
-config_node *cfg_add_class(config_obj *cfg, char *class);
-config_node *cfg_new_node();
-void cfg_add_child(config_node *parent, config_node *child);
+static void __int_cfg_set_single_value_as_string(config_obj *, char *, char *, char *);
+static void cfg_save(config_obj *);
+static void __int_cfg_remove_node(config_obj *, config_node *);
 
-void cfg_open_parse_file(config_obj *cfgo, FILE *fp)
+static void __int_cfg_do_special_cleanup(config_obj *cfg, config_node *node);
+static config_node *cfg_add_class(config_obj *, char *);
+static config_node *cfg_new_node(void);
+static void cfg_add_child(config_node *, config_node *);
+
+static void cfg_open_parse_file(config_obj *cfgo, FILE *fp)
 {
 	char buffer[1024];
 	int len = 0;
@@ -101,6 +113,8 @@ void cfg_open_parse_file(config_obj *cfgo, FILE *fp)
 				child->children = NULL;
 				cfg_add_child(cur, child);
 				multiple = child;
+				cfgo->total_size+= len+sizeof(config_node);
+
 			}
 			if(len==0)
 			{
@@ -130,6 +144,8 @@ void cfg_open_parse_file(config_obj *cfgo, FILE *fp)
 				new = cfg_new_node();
 				new->type = TYPE_ITEM;
 				new->name = g_strndup(buffer, len);	
+				cfgo->total_size+= len+sizeof(config_node);
+			/*	printf("node size: %i\n", sizeof(config_node));*/
 				/* Get value */
 				len = 0;
 				/* skip spaces */
@@ -164,6 +180,7 @@ void cfg_open_parse_file(config_obj *cfgo, FILE *fp)
 					c = fgetc(fp);
 				}while((c != '\n' || quote) && c != EOF && quote >= 0 && len < 1024);
 				new->value = g_strndup(buffer, len);
+				cfgo->total_size+= len;
 				if(multiple){
 					cfg_add_child(multiple,new);
 				}else{
@@ -192,9 +209,12 @@ config_obj *cfg_open(gchar *url)
 	{
 		return NULL;
 	}
-
+	cfgo->lock = g_mutex_new();
+	g_mutex_lock(cfgo->lock);
 	cfgo->url = g_strdup(url);
 	cfgo->root = NULL;
+	cfgo->total_size = sizeof(config_obj)+sizeof(&cfgo->lock)+strlen(cfgo->url);
+
 	if(g_file_test(cfgo->url, G_FILE_TEST_EXISTS))
 	{
 		FILE *fp = fopen(cfgo->url, "r");
@@ -204,7 +224,8 @@ config_obj *cfg_open(gchar *url)
 			fclose(fp);
 		}
 	}
-
+	debug_printf(DEBUG_INFO,"Config %s: allocated: %i\n", cfgo->url, cfgo->total_size);
+	g_mutex_unlock(cfgo->lock);
 	return cfgo;
 }
 
@@ -215,16 +236,24 @@ void cfg_close(config_obj *cfgo)
 	{
 		return;
 	}
+	debug_printf(DEBUG_INFO,"Closing config '%s' with %i bytes allocated\n", cfgo->url, cfgo->total_size);
+	g_mutex_lock(cfgo->lock);
 	if(cfgo->url != NULL)
 	{
-		g_free(cfgo->url);
+		cfgo->total_size-=strlen(cfgo->url);
+		cfg_free_string(cfgo->url);
 	}
-	while(cfgo->root)cfg_remove_node(cfgo,cfgo->root);
+	while(cfgo->root)__int_cfg_remove_node(cfgo,cfgo->root);
+	g_mutex_unlock(cfgo->lock);
+	cfgo->total_size-= sizeof(&cfgo->lock)+sizeof(config_obj);
+	g_mutex_free(cfgo->lock);
+	debug_printf(DEBUG_INFO,"Memory remaining: %i\n", cfgo->total_size);
 	g_free(cfgo);
+	cfgo = NULL;
 }
-config_node *cfg_new_node()
+static config_node *cfg_new_node()
 {
-	config_node *newnode = g_malloc0(sizeof(config_node));
+	config_node *newnode = g_slice_new(config_node);
 	newnode->type = TYPE_CATEGORY;
 	newnode->name = NULL; 
 	newnode->next = NULL;
@@ -233,13 +262,14 @@ config_node *cfg_new_node()
 	newnode->value = NULL;
 	return newnode;
 }
-config_node *cfg_add_class(config_obj *cfg, char *class)
+static config_node *cfg_add_class(config_obj *cfg, char *class)
 {
 	config_node *newnode = cfg_new_node();
 	newnode->type = TYPE_CATEGORY;
 	newnode->name = g_strdup(class);
 	newnode->value = NULL;
 	newnode->children  = NULL;
+	cfg->total_size += sizeof(config_node)+strlen(class);
 	if(cfg->root == NULL)
 	{
 		cfg->root = newnode;		
@@ -274,7 +304,7 @@ void cfg_add_child(config_node *parent, config_node *child)
 	}
 }
 
-void cfg_save_category(config_obj *cfg, config_node *node, FILE *fp)
+static void cfg_save_category(config_obj *cfg, config_node *node, FILE *fp)
 {
 	config_node *temp = NULL;
 	if(node == NULL)return;
@@ -315,10 +345,9 @@ void cfg_save_category(config_obj *cfg, config_node *node, FILE *fp)
 			fputs("\"\n",fp);
 		}
 	}
-
 }
 
-void cfg_save(config_obj *cfgo)
+static void cfg_save(config_obj *cfgo)
 {
 	if(cfgo == NULL)
 	{
@@ -342,7 +371,7 @@ void cfg_save(config_obj *cfgo)
 	return;
 }
 
-config_node *cfg_get_class(config_obj *cfg, char *class)
+static config_node *cfg_get_class(config_obj *cfg, char *class)
 {
 	config_node *node = cfg->root;
 	if(node == NULL) return NULL;
@@ -360,11 +389,11 @@ config_node *cfg_get_class(config_obj *cfg, char *class)
 	return NULL;
 }
 
-config_node *cfg_get_single_value(config_obj *cfg, char *class, char *key)
+static config_node *cfg_get_single_value(config_obj *cfg, char *class, char *key)
 {
 	/* take children */
-	config_node *cur = cfg_get_class(cfg, class);
-
+	config_node *cur = NULL;
+	cur = cfg_get_class(cfg, class);
 	if(cur == NULL || cur->children == NULL)
 	{
 		return NULL;
@@ -376,109 +405,165 @@ config_node *cfg_get_single_value(config_obj *cfg, char *class, char *key)
 			return cur;
 		}
 	}
-
 	return NULL;                                     	
 }
 
-void cfg_free_string(char *string)
+/*void cfg_free_string(char *string)
 {
 	if(string != NULL)
 	{
+		q_free(string);
 	}
 }
-
-char * cfg_get_single_value_as_string(config_obj *cfg, char *class, char *key)
+*/
+static char * __int_cfg_get_single_value_as_string(config_obj *cfg, char *class, char *key)
 {
 	config_node *cur = cfg_get_single_value(cfg, class,key);
 	if(cur != NULL)
 	{
 		if(cur->type == TYPE_ITEM)
 		{
-			return (char *)cur->value;
+			return g_strdup((char *)cur->value);
 		}
 	}
 	return NULL;
 }
 
-char * cfg_get_single_value_as_string_with_default(config_obj *cfg, char *class, char *key , char *def)
+char * cfg_get_single_value_as_string(config_obj *cfg, char *class, char *key)
 {
-	char *retv = cfg_get_single_value_as_string(cfg,class,key);
-	if(retv == NULL)
-	{
-		cfg_set_single_value_as_string(cfg,class,key,def);
-		retv = cfg_get_single_value_as_string(cfg,class,key);
-	}
+	char *retv = NULL;
+	g_mutex_lock(cfg->lock);
+	retv = __int_cfg_get_single_value_as_string(cfg,class,key);
+	g_mutex_unlock(cfg->lock);
 	return retv;
 }
 
-int cfg_get_single_value_as_int(config_obj *cfg, char *class, char *key)
+
+char * cfg_get_single_value_as_string_with_default(config_obj *cfg, char *class, char *key , char *def)
 {
-	config_node *cur = cfg_get_single_value(cfg, class,key);
+	char *retv = NULL;
+   	g_mutex_lock(cfg->lock);
+	retv = __int_cfg_get_single_value_as_string(cfg,class,key);
+	if(retv == NULL)
+	{
+		__int_cfg_set_single_value_as_string(cfg,class,key,def);
+		retv = __int_cfg_get_single_value_as_string(cfg,class,key);
+	}
+	g_mutex_unlock(cfg->lock);
+	return retv;
+}
+
+static int __int_cfg_get_single_value_as_int(config_obj *cfg, char *class, char *key)
+{
+	config_node *cur =NULL;
+	int retv = CFG_INT_NOT_DEFINED;
+
+	cur = cfg_get_single_value(cfg, class,key);
 	if(cur != NULL)
 	{
 		if(cur->type == TYPE_ITEM)
 		{
-			return (int)g_ascii_strtoull(cur->value,NULL,0);	
-
+			retv = (int)g_ascii_strtoull(cur->value,NULL,0);	
 		}
 	}
-	return CFG_INT_NOT_DEFINED;
+
+	return retv; 
+}
+int cfg_get_single_value_as_int(config_obj *cfg, char *class, char *key)
+{
+	int retv = 0;
+	g_mutex_lock(cfg->lock);
+	retv = __int_cfg_get_single_value_as_int(cfg, class, key);
+	g_mutex_unlock(cfg->lock);
+	return retv;
 }
 
 void cfg_set_single_value_as_int(config_obj *cfg, char *class, char *key, int value)
 {
-	gchar *	temp = g_strdup_printf("%i",value);
-	cfg_set_single_value_as_string(cfg,class,key,temp);
-	g_free(temp);
+	gchar *	temp = NULL;
+	g_mutex_lock(cfg->lock);
+	temp  = g_strdup_printf("%i",value);
+	__int_cfg_set_single_value_as_string(cfg,class,key,temp);
+	cfg_free_string(temp);
+	g_mutex_unlock(cfg->lock);
+}
+static void __int_cfg_set_single_value_as_int(config_obj *cfg, char *class, char *key, int value)
+{
+	gchar *	temp = NULL;
+	temp  = g_strdup_printf("%i",value);
+	__int_cfg_set_single_value_as_string(cfg,class,key,temp);
+	cfg_free_string(temp);
 }
 int cfg_get_single_value_as_int_with_default(config_obj *cfg, char *class, char *key, int def)
 {
-	int retv = cfg_get_single_value_as_int(cfg,class,key);
+	int retv = CFG_INT_NOT_DEFINED;
+	g_mutex_lock(cfg->lock);		
+	retv = __int_cfg_get_single_value_as_int(cfg,class,key);
 	if(retv == CFG_INT_NOT_DEFINED)
 	{
-		cfg_set_single_value_as_int(cfg,class,key,def);
-		retv = cfg_get_single_value_as_int(cfg,class,key);		
+		__int_cfg_set_single_value_as_int(cfg,class,key,def);
+		retv = __int_cfg_get_single_value_as_int(cfg,class,key);		
 	}
+	g_mutex_unlock(cfg->lock);		
 	return retv;
 }
 /* float */
-float cfg_get_single_value_as_float(config_obj *cfg, char *class, char *key)
+static float __int_cfg_get_single_value_as_float(config_obj *cfg, char *class, char *key)
 {
-	char * temp = cfg_get_single_value_as_string(cfg,class,key);
+	config_node *cur =NULL;
+	cur = cfg_get_single_value(cfg,class,key);
 	float result = 0;
-	if(temp == NULL)
+	if(cur == NULL)
 	{
 		return CFG_INT_NOT_DEFINED;
 	}
 	/* make it return an error */
-	result = g_ascii_strtod(temp,NULL);
+	result = g_ascii_strtod(cur->value,NULL);
 	return result;
 }
 
+float cfg_get_single_value_as_float(config_obj *cfg, char *class, char *key)
+{
+	float retv = 0;
+	g_mutex_lock(cfg->lock);
+	retv = __int_cfg_get_single_value_as_float(cfg,class,key);
+	g_mutex_unlock(cfg->lock);
+	return retv;
+}
+
 void cfg_set_single_value_as_float(config_obj *cfg, char *class, char *key, float value)
 {
-	char *value1 = g_strdup_printf("%f", value);
-	cfg_set_single_value_as_string(cfg,class,key,value1);
-	g_free(value1);
+	char *value1 = NULL;
+	g_mutex_lock(cfg->lock);
+	value1 = g_strdup_printf("%f", value);
+	__int_cfg_set_single_value_as_string(cfg,class,key,value1);
+	cfg_free_string(value1);
+	g_mutex_unlock(cfg->lock);
 }
 float cfg_get_single_value_as_float_with_default(config_obj *cfg, char *class, char *key, float def)
 {
-	float retv = cfg_get_single_value_as_float(cfg,class,key);
+	float retv = 0;
+	g_mutex_lock(cfg->lock);
+	retv = __int_cfg_get_single_value_as_float(cfg,class,key);
 	if(retv == CFG_INT_NOT_DEFINED)
 	{
-		cfg_set_single_value_as_float(cfg,class,key,def);
-		retv = cfg_get_single_value_as_float(cfg,class,key);		
+		char * value1 = g_strdup_printf("%f", def);
+		__int_cfg_set_single_value_as_string(cfg,class,key,value1);
+		cfg_free_string(value1);
+		retv = __int_cfg_get_single_value_as_float(cfg,class,key);		
 	}
+	g_mutex_unlock(cfg->lock);
 	/* make it return an error */
 	return retv;
 }
-void cfg_remove_node(config_obj *cfg, config_node *node)
+
+static void __int_cfg_remove_node(config_obj *cfg, config_node *node)
 {
 	if(node->type != TYPE_ITEM)
 	{
 		while(node->children)
 		{
-			cfg_remove_node(cfg,node->children);
+			__int_cfg_remove_node(cfg,node->children);
 		}
 	}
 	/*  only child, and I have a parent */
@@ -517,19 +602,45 @@ void cfg_remove_node(config_obj *cfg, config_node *node)
 			cfg->root = NULL;
 		}
 	}	
-	if(node->name) g_free(node->name);
-	if(node->value) g_free(node->value);
-	g_free(node);
+	cfg->total_size-= sizeof(config_node);
+	if(node->name){
+		cfg->total_size-=strlen(node->name);
+		 cfg_free_string(node->name);
+	}
+	if(node->value) {
+		cfg->total_size-=strlen(node->value);
+		cfg_free_string(node->value);
+	}
+	g_slice_free(config_node,node);
 }
 void cfg_del_single_value(config_obj *cfg, char *class, char *key)
 {
-	config_node *node = cfg_get_single_value(cfg,class,key);
-	if(node == NULL) return;
-	cfg_remove_node(cfg,node);
-	cfg_save(cfg);
+	config_node *node = NULL;
+	g_mutex_lock(cfg->lock);
+   	node = cfg_get_single_value(cfg,class,key);
+	if(node != NULL)
+	{
+		__int_cfg_remove_node(cfg,node);
+		cfg_save(cfg);
+	}
+	g_mutex_unlock(cfg->lock);
 }
+void cfg_remove_class(config_obj *cfg, char *class)
+{
+	config_node *node = NULL;
+	if(cfg == NULL || class == NULL)
+		return;
 
-void cfg_set_single_value_as_string(config_obj *cfg, char *class, char *key, char *value)
+	g_mutex_lock(cfg->lock);
+	node = cfg_get_class(cfg, class);
+	if(node)
+	{
+		__int_cfg_remove_node(cfg, node);
+	}
+	cfg_save(cfg);
+	g_mutex_unlock(cfg->lock);
+}
+static void __int_cfg_set_single_value_as_string(config_obj *cfg, char *class, char *key, char *value)
 {
 	config_node *newnode = cfg_get_single_value(cfg,class,key);
 	if(newnode == NULL)
@@ -542,19 +653,31 @@ void cfg_set_single_value_as_string(config_obj *cfg, char *class, char *key, cha
 		}
 		newnode = cfg_new_node();
 		newnode->name = g_strdup(key);
+		cfg->total_size+=sizeof(config_node)+strlen(key);
 		cfg_add_child(node,newnode);
 
 	}	
 	newnode->type = TYPE_ITEM;
-	if(newnode->value) g_free(newnode->value);
+	if(newnode->value){
+		cfg->total_size-= strlen(newnode->value);
+		cfg_free_string(newnode->value);
+	}
 	newnode->value = g_strdup(value);	
+	cfg->total_size += strlen(value);
 	cfg_save(cfg);
 }
 
+void cfg_set_single_value_as_string(config_obj *cfg, char *class, char *key, char *value)
+{
+	g_mutex_lock(cfg->lock);
+	__int_cfg_set_single_value_as_string(cfg, class,key,value);
+	g_mutex_unlock(cfg->lock);
+}
+
 /* multiple values */
 
 
-config_node *cfg_get_multiple_value(config_obj *cfg, char *class, char *key, char* id)
+static config_node *cfg_get_multiple_value(config_obj *cfg, char *class, char *key, char* id)
 {
 	config_node *node = cfg_get_single_value(cfg,class,key);
 	if(node == NULL){
@@ -581,21 +704,31 @@ config_node *cfg_get_multiple_value(config_obj *cfg, char *class, char *key, cha
 
 void cfg_del_multiple_value(config_obj *cfg, char *class, char *key,char *id)
 {
-	config_node *cur = cfg_get_multiple_value(cfg, class,key,id);
+	config_node *cur = NULL;
+	g_mutex_lock(cfg->lock);
+	cur = cfg_get_multiple_value(cfg, class,key,id);
 	if(cur != NULL)
 	{
-		cfg_remove_node(cfg,cur);
+		__int_cfg_remove_node(cfg,cur);
 		cfg_save(cfg);
 	}
+	g_mutex_unlock(cfg->lock);
 }
 
 void cfg_set_multiple_value_as_string(config_obj *cfg, char *class, char *key, char *id, char *value)
 {
-	config_node *cur = cfg_get_multiple_value(cfg, class,key,id);
+	config_node *cur = NULL;
+	g_mutex_lock(cfg->lock);
+	cur = cfg_get_multiple_value(cfg, class,key,id);
 	if(cur != NULL)
 	{
-		if(cur->value) g_free(cur->value);
+		if(cur->value){
+			cfg->total_size -= strlen(cur->value);
+			cfg_free_string(cur->value);
+		}
+
 		cur->value = g_strdup(value);
+		cfg->total_size += strlen(cur->value);
 	}
 	else {
 		config_node *node = cfg_get_single_value(cfg,class,key);
@@ -609,6 +742,7 @@ void cfg_set_multiple_value_as_string(config_obj *cfg, char *class, char *key, c
 			cur->name = g_strdup(key);
 			cur->type = TYPE_ITEM_MULTIPLE;
 			cfg_add_child(node,cur);
+			cfg->total_size+=sizeof(config_node)+strlen(key);
 			node = cur;
 		}
 		if(node->type != TYPE_ITEM_MULTIPLE) return;
@@ -616,32 +750,36 @@ void cfg_set_multiple_value_as_string(config_obj *cfg, char *class, char *key, c
 		cur->type = TYPE_ITEM;
 		cur->name = g_strdup(id);
 		cur->value = g_strdup(value);
+		cfg->total_size+=sizeof(config_node)+strlen(id)+strlen(value);;
 		cfg_add_child(node,cur);
 		cfg_save(cfg);
 	}
+	g_mutex_unlock(cfg->lock);
 }
 
 conf_mult_obj * cfg_get_multiple_as_string(config_obj *cfg, char *class, char *key)
 {
-	config_node *cur = cfg_get_single_value(cfg, class,key);
+	conf_mult_obj *list = NULL;
+	config_node *cur = NULL;
+	g_mutex_lock(cfg->lock);
+	cur = cfg_get_single_value(cfg, class,key);
 	if(cur && cur->type == TYPE_ITEM_MULTIPLE && cur->children != NULL)
 	{
-		conf_mult_obj *list = NULL;
 		cur = cur->children;	
 		/* get first */
 		while(cur->prev != NULL) cur = cur->prev;
 		do {
-			conf_mult_obj *temp= g_malloc0(sizeof(conf_mult_obj));
-			temp->value = cur->value;
-			temp->key = cur->name;
+			conf_mult_obj *temp= g_slice_new0(conf_mult_obj);
+			temp->value = g_strdup(cur->value);
+			temp->key = g_strdup(cur->name);
 			temp->next = list;
 			if(temp->next) temp->next->prev = temp;
 			list = temp;
 			cur = cur->next;
 		}while(cur != NULL);		
-		return list;
 	}
-	return NULL;
+	g_mutex_unlock(cfg->lock);
+	return list;
 }
 void cfg_free_multiple(conf_mult_obj *data)
 {
@@ -654,14 +792,16 @@ void cfg_free_multiple(conf_mult_obj *data)
 	}
 	while(list != NULL)
 	{
+		if(list->value)	cfg_free_string(list->value);
+		if(list->key)	cfg_free_string(list->key);  		
 		if(list->next != NULL)
 		{
-			if(list->prev)g_free(list->prev);
+			if(list->prev)g_slice_free(conf_mult_obj,list->prev);
 			list = list->next;
 		}
 		else{
-			g_free(list->prev);
-			g_free(list);
+			g_slice_free(conf_mult_obj,list->prev);
+			g_slice_free(conf_mult_obj,list);
 			list = NULL;
 		}
 	}
@@ -673,15 +813,15 @@ conf_mult_obj *cfg_get_class_list(config_obj *data)
 	config_node *root = NULL;
 	if(!data) return NULL;
 	if(!data->root) return NULL;
-
+	g_mutex_lock(data->lock);
 	root = data->root;
 	while(root->prev != NULL) root = root->prev;
 	do {
 		if(root->type == TYPE_CATEGORY)
 		{
-			conf_mult_obj *temp= g_malloc0(sizeof(conf_mult_obj));
+			conf_mult_obj *temp= g_slice_new0(conf_mult_obj);
 			temp->value = NULL;
-			temp->key = root->name;
+			temp->key = g_strdup(root->name);
 			temp->next = list;
 			if(temp->next) temp->next->prev = temp;
 			list = temp;
@@ -689,6 +829,7 @@ conf_mult_obj *cfg_get_class_list(config_obj *data)
 		root=root->next;
 	}while(root!= NULL);		
 	while(list->prev != NULL) list = list->prev;
+	g_mutex_unlock(data->lock);
 	return list;
 }
 
@@ -698,18 +839,21 @@ conf_mult_obj *cfg_get_key_list(config_obj *data,char *class)
 	config_node *root = NULL;
 	if(data == NULL) return NULL;
 	if(data->root == NULL) return NULL;
-
+	g_mutex_lock(data->lock);
 	root = cfg_get_class(data,class);
-	if(!root) return NULL;
+	if(!root || !root->children)
+	{
+		g_mutex_unlock(data->lock);
+		return NULL;
+	}
 	root = root->children;
-	if(!root) return NULL;
 	while(root->prev != NULL) root = root->prev;
 	do {
 		if(root->type == TYPE_ITEM)
 		{
-			conf_mult_obj *temp= g_malloc0(sizeof(conf_mult_obj));
-			temp->value = root->value;
-			temp->key = root->name;
+			conf_mult_obj *temp= g_slice_new0(conf_mult_obj);
+			temp->value = g_strdup(root->value);
+			temp->key = g_strdup(root->name);
 			temp->next = list;
 			if(temp->next) temp->next->prev = temp;
 			list = temp;
@@ -717,6 +861,49 @@ conf_mult_obj *cfg_get_key_list(config_obj *data,char *class)
 		root=root->next;
 	}while(root!= NULL);
 	while(list->prev != NULL) list = list->prev;
+	g_mutex_unlock(data->lock);
 	return list;
 }
 
+static void __int_cfg_do_special_cleanup(config_obj *cfg, config_node *node)
+{
+	config_node *root = NULL;
+	if(!cfg || !cfg->root)
+		return;
+	if(node == NULL)
+		root = cfg->root;
+	else
+		root = node;
+	if(!root)return;
+	while(root)
+	{
+		if(root->type == TYPE_CATEGORY)
+		{
+			if(root->children)
+				__int_cfg_do_special_cleanup(cfg, root->children);		
+		}	
+		else if(root->type == TYPE_ITEM)
+		{
+			if(root->value == NULL || root->value[0] == '\0')
+			{
+				config_node *node = root;
+				root = node->prev;
+				if(!root)
+					root = node->next;
+				__int_cfg_remove_node(cfg, node);
+				if(!root) return;
+			}
+		}
+		root = root->next;
+	}
+}
+void cfg_do_special_cleanup(config_obj *cfg)
+{
+	if(!cfg || !cfg->root)
+		return;
+
+	g_mutex_lock(cfg->lock);
+	__int_cfg_do_special_cleanup(cfg, NULL);
+	cfg_save(cfg);
+	g_mutex_unlock(cfg->lock);
+}
