@@ -1,7 +1,7 @@
 #ifndef __MAIN_H__
 
 #define __MAIN_H__
-
+#include <config.h>
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
@@ -16,9 +16,6 @@
 #define _(String) String
 #endif
 
-
-
-
 #include "config-defaults.h"
 #include <libmpd/libmpd.h>
 #include <libmpd/libmpdclient.h>
@@ -26,42 +23,53 @@
 #include <glade/glade.h>
 #include "config1.h"
 #include "plugin.h"
-#include "player.h"
+#include "metadata.h"
 #include "playlist3.h"
+#include "playlist-list.h"
 #include "mpdinteraction.h"
-#include "cover-art.h"
-#include "qthread.h"
+#include "tray-icon.h"
+
+/**
+ * Some gobjects
+ */
+#include "gmpc-connection.h"
+#include "gmpc-metaimage.h"
+
+extern GmpcConnection *gmpcconn;
+
+extern int gmpc_connected;
+
+extern GtkTreeModel *playlist;
 
 /* the config object */
 extern config_obj *config;
+extern config_obj *profiles;
+
 /* the plugin list */
 extern gmpcPlugin **plugins;
 /* num of plugins */
 extern int num_plugins;
 
-
 /* the plugins :D */
+
+
+
+
 extern gmpcPlugin connection_plug;
+/*
 extern gmpcPlugin about_plug;
+*/
 extern gmpcPlugin playlist_plug;
+
 extern gmpcPlugin cover_art_plug;
 
-#ifdef ENABLE_TRAYICON
 extern gmpcPlugin tray_icon_plug;
-#endif
-
+extern gmpcPlugin info3_plugin;
+extern gmpcPlugin metab_plugin;
+extern gmpcPlugin url_plugin; 
 /** main.c **/
 extern GladeXML *xml_main_window;
 extern MpdObj *connection;
-typedef struct
-{
-	/* playlist number this is to check if the playlist changed */
-	long long playlist_id;
-	int playlist_length;
-	int playlist_playtime;
-	/* the current song */
-	int old_pos;
-} internal_data;
 
 enum{
 	TIME_FORMAT_ELAPSED,
@@ -69,18 +77,6 @@ enum{
 	TIME_FORMAT_PERCENTAGE
 };
 
-extern internal_data info;
-extern guint update_timeout;
-int update_interface();
-
-/* mpdinteraction.c*/
-int update_mpd_status();
-
-void create_preferences_window();
-
-/* id3info.c*/
-void call_id3_window(int song);
-
 /* main.h*/
 void main_trigger_update();
 
@@ -88,17 +84,15 @@ void pl3_highlight_song_change ();
 char * edit_song_markup(char *format);
 
 gboolean playlist_filter_func(GtkTreeModel *model, GtkTreeIter *iter);
-void id3_status_update();
 void call_id3_window_song(mpd_Song *songstr);
 void playlist_changed(MpdObj *mi);
 
-void main_quit();
+void main_quit(void);
 
 /*
  * functions to get patch to different files.
  * This is needed to make the windows port work.
  */
-char *gmpc_get_full_image_path(char *filename);
 char *gmpc_get_full_glade_path(char *filename);
 
 /* plugin */
@@ -106,8 +100,37 @@ void plugin_load_dir(gchar *path);
 void plugin_add(gmpcPlugin *plug, int plugin);
 int plugin_get_pos(int id);
 
+void show_error_message(gchar *string, int block);
 
+/* This is over here because of it's need for xml_error_window, this should probably
+ * be moved to mpdinteraction.c
+ */
+void connect_callback(MpdObj *mi);
+
+/** Handle status changed */
+void   GmpcStatusChangedCallback(MpdObj *mi, ChangedStatusType what, void *userdata);
+
+/**
+ * Metadata 
+ */
+
+void meta_data_add_plugin(gmpcPlugin *plug);
+/**
+ * TODO move this 
+ */
+void url_start_real(const gchar *url);
+
+/*
+ * functions to get patch to different files.
+ * This is needed to make the windows port work.
+ */
+char *gmpc_get_full_image_path(char *filename);
+
+void preferences_show_pref_window(int plugin_id);
+
+/**
+ * Help functions
+ */
+#define q_free(a) g_free(a);a=NULL;
 
-void show_error_message(gchar *string);
-int cover_art_edit_cover(gchar *artist, gchar *album);
 #endif
