@@ -1,5 +1,5 @@
 /*
- *Copyright (C) 2004-2005 Qball Cow <Qball@qballcow.nl>
+ *Copyright (C) 2004-2007 Qball Cow <qball@sarine.nl>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -29,16 +29,29 @@
 #include "main.h"
 #include "misc.h"
 #include "playlist3.h"
-#include "playlist3-current-playlist-browser.h"
+#include "playlist3-artist-browser.h"
 #include "config1.h"
 #include "TreeSearchWidget.h"
 
-void pl3_artist_browser_row_activated(GtkTreeView *tree, GtkTreePath *tp);
-void pl3_artist_browser_show_info();
-void pl3_artist_browser_button_release_event(GtkWidget *but, GdkEventButton *event);
-void pl3_artist_browser_add_selected();
-void pl3_artist_browser_replace_selected();
-int pl3_artist_browser_playlist_key_press(GtkWidget *tree, GdkEventKey *event);
+static void pl3_artist_browser_add(GtkWidget *cat_tree);
+static void pl3_artist_browser_fill_tree(GtkWidget *, GtkTreeIter *);
+static void pl3_artist_browser_category_selection_changed(GtkWidget *,GtkTreeIter *);
+static void pl3_artist_browser_selected(GtkWidget *);
+static void pl3_artist_browser_unselected(GtkWidget *);
+static int pl3_artist_browser_cat_popup(GtkWidget *, int ,GtkWidget *, GdkEventButton *);
+static void pl3_artist_browser_category_key_press(GtkWidget *, GdkEventKey *, int);
+static void pl3_artist_browser_disconnect(void);
+static int pl3_artist_browser_add_go_menu(GtkWidget *);
+static void pl3_artist_browser_row_activated(GtkTreeView *, GtkTreePath *);
+static void pl3_artist_browser_button_release_event(GtkWidget *but, GdkEventButton *event);
+static void pl3_artist_browser_add_selected(void);
+static void pl3_artist_browser_replace_selected(void);
+static int pl3_artist_browser_playlist_key_press(GtkWidget *tree, GdkEventKey *event);
+static void pl3_artist_browser_connection_changed(MpdObj *mi, int connect, gpointer data);
+static int pl3_artist_browser_key_press_event(GtkWidget *mw, GdkEventKey *event, int type);
+static void pl3_artist_browser_status_changed(MpdObj *mi,ChangedStatusType what, void *data);
+
+static void pl3_artist_browser_reupdate(void);
 
 enum{
 	PL3_AB_ARTIST,
@@ -49,7 +62,39 @@ enum{
 	PL3_AB_ICON,
 	PL3_AB_ROWS
 };
+extern GladeXML *pl3_xml;
 
+/**
+ * Plugin structure
+ */
+gmpcPlBrowserPlugin artist_browser_gbp = {
+	pl3_artist_browser_add,
+	pl3_artist_browser_selected,
+	pl3_artist_browser_unselected,
+	pl3_artist_browser_category_selection_changed,
+	pl3_artist_browser_fill_tree,
+	pl3_artist_browser_cat_popup,
+	pl3_artist_browser_category_key_press,
+	pl3_artist_browser_add_go_menu,
+	pl3_artist_browser_key_press_event
+};
+
+gmpcPlugin artist_browser_plug = {
+	"Artist Browser",
+	{1,1,1},
+	GMPC_PLUGIN_PL_BROWSER,
+	0,
+	NULL,			                /* path*/
+	NULL,			                /* init */
+        NULL,                                   /* Destroy */
+	&artist_browser_gbp,		        /* Browser */
+	pl3_artist_browser_status_changed,	/* status changed */
+	pl3_artist_browser_connection_changed, 	/* connection changed */
+	NULL,		                        /* Preferences */
+	NULL,			                /* MetaData */
+	NULL,			                /* Get enable */
+	NULL			                /* Set Enable */
+};
 
 /* internal */
 GtkWidget *pl3_ab_tree = NULL;
@@ -57,33 +102,11 @@ GtkListStore *pl3_ab_store = NULL;
 GtkWidget *pl3_ab_vbox = NULL;
 GtkWidget *pl3_ab_tree_search = NULL;
 
-GtkWidget *pl3_cat_tree = NULL; /* the left pane tree */
 
 
 GtkTreeRowReference *pl3_ab_tree_ref = NULL;
 
-
-void pl3_artist_browser_dbase_updated()
-{
-	if(pl3_ab_tree_ref){
-		GtkTreeIter parent, child;
-		GtkTreeModel *model = gtk_tree_row_reference_get_model(pl3_ab_tree_ref);
-		GtkTreePath *path = gtk_tree_row_reference_get_path(pl3_ab_tree_ref);
-		if(!path) return;
-		if(gtk_tree_model_get_iter(model, &parent, path)){
-			while(gtk_tree_model_iter_children(model, &child,&parent))
-			{
-				gtk_tree_store_remove(GTK_TREE_STORE(model), &child);
-			}
-		}
-		/* add fake, and set not-openend bit */
-		gtk_tree_store_append(pl3_tree, &child, &parent);
-		gtk_tree_store_set(GTK_TREE_STORE(model), &parent, PL3_CAT_PROC,FALSE,-1);
-
-	}
-}
-
-int pl3_artist_browser_button_press_event(GtkTreeView *tree, GdkEventButton *event)
+static int pl3_artist_browser_button_press_event(GtkTreeView *tree, GdkEventButton *event)
 {
 	GtkTreeSelection *sel = gtk_tree_view_get_selection(tree);
 	if(event->button != 3 || gtk_tree_selection_count_selected_rows(sel) < 2|| !mpd_check_connected(connection))
@@ -106,18 +129,18 @@ static void pl3_artist_browser_search_activate()
 	}
 }
 
-void pl3_artist_browser_init()
+static void pl3_artist_browser_init()
 {
 	GtkWidget *pl3_ab_sw = NULL;
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column = NULL;
 	GValue value;
 	pl3_ab_store = gtk_list_store_new (PL3_AB_ROWS, 
-			GTK_TYPE_STRING, /* PL3_AB_ARTIST */
-			GTK_TYPE_STRING, /* PL3_AB_ALBUM  */
-			GTK_TYPE_STRING, /* PL3_AB_FILE */
-			GTK_TYPE_INT,	 /* PL3_AB_TYPE  */
-			GTK_TYPE_STRING, /* PL3_AB_TITLE */
+			G_TYPE_STRING, /* PL3_AB_ARTIST */
+			G_TYPE_STRING, /* PL3_AB_ALBUM  */
+			G_TYPE_STRING, /* PL3_AB_FILE */
+			G_TYPE_INT,	 /* PL3_AB_TYPE  */
+			G_TYPE_STRING, /* PL3_AB_TITLE */
 			GDK_TYPE_PIXBUF); /* icon type */
 
 
@@ -173,19 +196,14 @@ void pl3_artist_browser_init()
 }
 
 
-/****************************************************************************************
- *  ARTIST BROWSER									*
- *  When mpd 0.12 is stable this function is deprecated, but needed for pre-0.12 mpd's  *
- ****************************************************************************************/
-
-void pl3_artist_browser_add()
+static void pl3_artist_browser_add(GtkWidget *cat_tree)
 {
 	GtkTreePath *path;
 	GtkTreeIter iter,child;
 	gtk_tree_store_append(pl3_tree, &iter, NULL);
 	gtk_tree_store_set(pl3_tree, &iter, 
-			PL3_CAT_TYPE, PL3_BROWSE_ARTIST,
-			PL3_CAT_TITLE, "Browse Artists",        	
+			PL3_CAT_TYPE, artist_browser_plug.id,
+			PL3_CAT_TITLE, _("Browse Artists"),
 			PL3_CAT_INT_ID, "",
 			PL3_CAT_ICON_ID, "media-artist",
 			PL3_CAT_PROC, FALSE,
@@ -205,8 +223,9 @@ void pl3_artist_browser_add()
 	}
 }
 
-void pl3_artist_browser_cover_art_fetched(mpd_Song *song, GtkTreeRowReference *ref)
+static void pl3_artist_browser_cover_art_fetched(mpd_Song *song, MetaDataResult ret, char *coverpath,gpointer data)
 {
+	GtkTreeRowReference *ref = data;
 	if(song == NULL || ref == NULL) return;
 	else
 	{
@@ -214,31 +233,46 @@ void pl3_artist_browser_cover_art_fetched(mpd_Song *song, GtkTreeRowReference *r
 		GtkTreePath *path = gtk_tree_row_reference_get_path(ref);
 		if(path)
 		{
-
 			if(gtk_tree_model_get_iter(GTK_TREE_MODEL(pl3_ab_store), &iter, path))
 			{
-				char *coverpath = NULL;
-				int ret = 0;
-				ret = cover_art_fetch_image_path(song, &coverpath); 
-				if(ret == COVER_ART_OK_LOCAL)
+				if(ret == META_DATA_AVAILABLE)
 				{
-					int size = cfg_get_single_value_as_int_with_default(config, "cover-art", "browser-size",80);
+					int size = cfg_get_single_value_as_int_with_default(config, "cover-art", "browser-size",64);
 					GdkPixbuf *pb = gdk_pixbuf_new_from_file_at_size(coverpath,size,size,NULL);
+					screenshot_add_border(&pb);
 					gtk_list_store_set(pl3_ab_store,&iter, PL3_AB_ICON, pb, -1);
-					g_object_unref(pb);
-
+					if(pb)g_object_unref(pb);
+					gtk_tree_row_reference_free(ref);
+				}
+				else if(ret == META_DATA_FETCHING)
+				{
+					int size = cfg_get_single_value_as_int_with_default(config, "cover-art", "browser-size",64);
+					GdkPixbuf *pb2, *pb= gtk_widget_render_icon(GTK_WIDGET(pl3_ab_tree), "media-loading-cover",-1, NULL);
+					pb2 = gdk_pixbuf_scale_simple(pb,size, size,GDK_INTERP_BILINEAR);
+					gtk_list_store_set(pl3_ab_store,&iter, PL3_AB_ICON, pb2, -1);
+					if(pb)g_object_unref(pb);
+					if(pb2)g_object_unref(pb2);
+				}
+				else
+				{
+					int size = cfg_get_single_value_as_int_with_default(config, "cover-art", "browser-size",64);
+					GdkPixbuf *pb2,*pb = gtk_widget_render_icon(GTK_WIDGET(pl3_ab_tree),"media-no-cover", -1, NULL);
+					pb2 = gdk_pixbuf_scale_simple(pb,size, size,GDK_INTERP_BILINEAR);
+					gtk_list_store_set(pl3_ab_store,&iter, PL3_AB_ICON, pb2, -1);
+					if(pb)g_object_unref(pb);
+					if(pb2)g_object_unref(pb2);
+					gtk_tree_row_reference_free(ref);
 				}
-				if(coverpath)g_free(coverpath);
 			}
 			gtk_tree_path_free(path);
 		}
 	}
-	gtk_tree_row_reference_free(ref);
+
 }
 
 
 
-long unsigned pl3_artist_browser_view_folder(GtkTreeIter *iter_cat)
+static long unsigned pl3_artist_browser_view_folder(GtkTreeIter *iter_cat)
 {
 	char *artist, *string;
 	GtkTreeIter iter;
@@ -247,12 +281,12 @@ long unsigned pl3_artist_browser_view_folder(GtkTreeIter *iter_cat)
 	long unsigned time =0;
 
 	if(pl3_ab_tree == NULL || pl3_ab_store == NULL) return 0;
-	
-	gtk_tree_model_get(GTK_TREE_MODEL(pl3_tree), iter_cat, 2 , &artist, 1,&string, -1);
 	if (!mpd_check_connected(connection))
 		return 0;
 
-
+	/**
+	 * Get Depth
+	 */
 	path = gtk_tree_model_get_path(GTK_TREE_MODEL(pl3_tree), iter_cat);
 	if(path == NULL)
 	{
@@ -260,10 +294,14 @@ long unsigned pl3_artist_browser_view_folder(GtkTreeIter *iter_cat)
 	}
 	depth = gtk_tree_path_get_depth(path) -1;
 	gtk_tree_path_free(path);
+
+	gtk_tree_model_get(GTK_TREE_MODEL(pl3_tree), iter_cat, 2 , &artist, 1,&string, -1);
+
+
 	if(artist == NULL || string == NULL)
 	{
-		if(artist) g_free(artist);
-		if(string) g_free(string);
+		if(artist) q_free(artist);
+		if(string) q_free(string);
 		return 0;
 	}
 	if(depth == 0)
@@ -283,33 +321,49 @@ long unsigned pl3_artist_browser_view_folder(GtkTreeIter *iter_cat)
 					PL3_AB_ICON,pb,
 					-1);
 			if(pb)g_object_unref(pb);
+			/* To Damn Slow 
+			if(strlen(data->tag))
+			{
+				GtkTreePath *path = gtk_tree_model_get_path(GTK_TREE_MODEL(pl3_ab_store), &iter);
+				GtkTreeRowReference* rowref = gtk_tree_row_reference_new(GTK_TREE_MODEL(pl3_ab_store),path);		
+				mpd_Song *song = mpd_newSong();
+				song->artist = g_strdup(data->tag);
+				song->album = NULL; 
+				meta_data_get_path_callback(song, META_ARTIST_ART, pl3_artist_browser_cover_art_fetched, rowref);
+				mpd_freeSong(song);	
+				gtk_tree_path_free(path);
+			}
+			*/
 			data = mpd_data_get_next(data);
 		}
 		gtk_tree_view_set_model(GTK_TREE_VIEW(pl3_ab_tree), GTK_TREE_MODEL(pl3_ab_store));
-
-		g_free(artist);
-		g_free(string);
-
-		return 0;
 	}
-	if(depth == 1)
+	else if(depth == 1)
 	{
 		int albums = 0;
-		MpdData *data = mpd_database_get_albums(connection,artist);
+		GdkPixbuf *pb = NULL;
+		MpdData *data = NULL;
+		/**
+		 * Set "Up" entry
+		 */
+		pb = gtk_widget_render_icon(pl3_ab_tree, "gtk-go-up",GTK_ICON_SIZE_MENU,NULL);
+		gtk_list_store_append (pl3_ab_store, &iter);
+		gtk_list_store_set (pl3_ab_store,&iter,
+				PL3_AB_ARTIST, NULL,
+				PL3_AB_ALBUM, NULL,
+				PL3_AB_TYPE, PL3_ENTRY_DIR_UP,
+				PL3_AB_TITLE, "..",
+				PL3_AB_ICON,pb,
+				-1);
+		if(pb) g_object_unref(pb);	
+
+		data = mpd_database_get_albums(connection,artist);
 		while(data != NULL){
-			char *coverpath = NULL;
-			int ret = 0;
-			GdkPixbuf *pb = NULL;
-			ret = cover_art_fetch_image_path_aa(artist,data->tag, &coverpath);
-			if(ret == COVER_ART_OK_LOCAL)
-			{
-				int size = cfg_get_single_value_as_int_with_default(config, "cover-art","browser-size", 80);
-				pb = gdk_pixbuf_new_from_file_at_size(coverpath,size,size,NULL);
-			}	
-			else
-			{
-				pb = gtk_widget_render_icon(pl3_ab_tree, "media-album", GTK_ICON_SIZE_MENU,NULL);
-			}
+			GtkTreePath *path = NULL;
+			GtkTreeRowReference* rowref = NULL;
+			mpd_Song *song = NULL; 
+
+			pb = gtk_widget_render_icon(pl3_ab_tree, "media-no-cover",-1/* GTK_ICON_SIZE_MENU*/,NULL);
 			gtk_list_store_append (pl3_ab_store, &iter);
 			gtk_list_store_set (pl3_ab_store,&iter,
 					PL3_AB_ARTIST, artist,
@@ -318,17 +372,21 @@ long unsigned pl3_artist_browser_view_folder(GtkTreeIter *iter_cat)
 					PL3_AB_TITLE, data->tag,
 					PL3_AB_ICON,pb,
 					-1);
+			if(pb)g_object_unref(pb);
 
-			if(ret == COVER_ART_NOT_FETCHED)
-			{
-				GtkTreePath *path = gtk_tree_model_get_path(GTK_TREE_MODEL(pl3_ab_store), &iter);
-				GtkTreeRowReference* rowref = gtk_tree_row_reference_new(GTK_TREE_MODEL(pl3_ab_store),path);		
-				cover_art_fetch_image_aa(artist,data->tag,
-						(CoverArtCallback)pl3_artist_browser_cover_art_fetched,rowref);
-				gtk_tree_path_free(path);
-			}
-			g_object_unref(pb);
-			if(coverpath) g_free(coverpath);
+			/**
+			 * create song and request metadata 
+			 */
+			path = gtk_tree_model_get_path(GTK_TREE_MODEL(pl3_ab_store), &iter);
+			rowref = gtk_tree_row_reference_new(GTK_TREE_MODEL(pl3_ab_store),path);		
+			song = mpd_newSong();                                                                     			
+			song->artist = g_strdup(artist);
+			song->album = g_strdup(data->tag);
+			meta_data_get_path_callback(song, META_ALBUM_ART, pl3_artist_browser_cover_art_fetched, rowref);
+			mpd_freeSong(song);	
+			gtk_tree_path_free(path);
+
+			
 			data = mpd_data_get_next(data);
 		}
 
@@ -339,8 +397,7 @@ long unsigned pl3_artist_browser_view_folder(GtkTreeIter *iter_cat)
 		{
 			if(data->type == MPD_DATA_TYPE_SONG)
 			{
-				if (data->song->album == NULL
-						|| strlen (data->song->album) == 0)
+				if (data->song->album == NULL || strlen (data->song->album) == 0)
 				{
 					gchar buffer[1024];
 					GdkPixbuf *pb = gtk_widget_render_icon(pl3_ab_tree, "media-audiofile", GTK_ICON_SIZE_MENU,NULL);
@@ -364,7 +421,7 @@ long unsigned pl3_artist_browser_view_folder(GtkTreeIter *iter_cat)
 							PL3_AB_TYPE,	PL3_ENTRY_SONG,
 							PL3_AB_ICON,	pb,
 							-1);
-					g_object_unref(pb);
+					if(pb)g_object_unref(pb);
 				}
 				else albums++;
 			}
@@ -381,28 +438,37 @@ long unsigned pl3_artist_browser_view_folder(GtkTreeIter *iter_cat)
 	}
 	else if(depth ==2)
 	{
+		char *markdata = cfg_get_single_value_as_string_with_default(config, "playlist", "browser_markup",DEFAULT_MARKUP_BROWSER);
+		GdkPixbuf *pb = NULL;
+		MpdData *data = NULL;
+		/**
+		 * Set "Up" entry
+		 */
+		pb = gtk_widget_render_icon(pl3_ab_tree, "gtk-go-up",GTK_ICON_SIZE_MENU,NULL);
+		gtk_list_store_append (pl3_ab_store, &iter);                                  		
+		gtk_list_store_set (pl3_ab_store,&iter,
+				PL3_AB_ARTIST, NULL,
+				PL3_AB_ALBUM, NULL,
+				PL3_AB_TYPE, PL3_ENTRY_DIR_UP,
+				PL3_AB_TITLE, "..",
+				PL3_AB_ICON,pb,
+				-1);
+		if(pb) g_object_unref(pb);	
 		/* artist and album is selected */
-		MpdData *data = mpd_database_find(connection,MPD_TABLE_ALBUM, string, TRUE);
+		data = mpd_database_find(connection,MPD_TABLE_ALBUM, string, TRUE);
 		while (data != NULL)
 		{
 			if(data->type == MPD_DATA_TYPE_SONG)
 			{
-				if (data->song->artist!= NULL
-						&& !g_utf8_collate (data->song->artist, artist))
+				if (data->song->artist!= NULL && !g_utf8_collate (data->song->artist, artist))
 				{
 					gchar buffer[1024];
-					GdkPixbuf *pb = gtk_widget_render_icon(pl3_ab_tree, "media-audiofile", GTK_ICON_SIZE_MENU,NULL);
-					char *markdata = cfg_get_single_value_as_string_with_default(config, "playlist", "browser_markup",DEFAULT_MARKUP_BROWSER);
+					pb = gtk_widget_render_icon(pl3_ab_tree, "media-audiofile", GTK_ICON_SIZE_MENU,NULL);
 					mpd_song_markup(buffer, 1024,markdata,data->song);
-					cfg_free_string(markdata);
 					if(data->song->time != MPD_SONG_NO_TIME)
 					{
 						time += data->song->time;
 					}
-					if(data->song->file == NULL)
-					{
-						debug_printf(DEBUG_WARNING,"pl3_browser_view_folder: crap mpdSong has no file attribute.\n");
-					}
 					gtk_list_store_append (pl3_ab_store, &iter);
 					gtk_list_store_set (pl3_ab_store, &iter,
 							PL3_AB_TITLE, buffer,
@@ -412,22 +478,22 @@ long unsigned pl3_artist_browser_view_folder(GtkTreeIter *iter_cat)
 							PL3_AB_TYPE, PL3_ENTRY_SONG,
 							PL3_AB_ICON,pb,
 							-1);
-					g_object_unref(pb);
-
+					if(pb)g_object_unref(pb);
 				}
 			}
 			data = mpd_data_get_next(data);
 		}
+		cfg_free_string(markdata);
 
 	}
-	g_free(artist);
-	g_free(string);
+	q_free(artist);
+	q_free(string);
 
 	return time;
 }
 
 
-void pl3_artist_browser_fill_tree(GtkTreeIter *iter)
+void pl3_artist_browser_fill_tree(GtkWidget *tree, GtkTreeIter *iter)
 {
 	char *artist, *alb_artist;
 	int depth =0;
@@ -459,7 +525,7 @@ void pl3_artist_browser_fill_tree(GtkTreeIter *iter)
 		{
 			gtk_tree_store_append (pl3_tree, &child, iter);
 			gtk_tree_store_set (pl3_tree, &child,
-					0, PL3_BROWSE_ARTIST,
+					0, artist_browser_plug.id,
 					1, data->tag, /* the field */
 					2, data->tag, /* the artist name, if(1 and 2 together its an artist field) */
 					3, "media-artist",
@@ -482,7 +548,7 @@ void pl3_artist_browser_fill_tree(GtkTreeIter *iter)
 		while(data != NULL){
 			gtk_tree_store_append (pl3_tree, &child, iter);
 			gtk_tree_store_set (pl3_tree, &child,
-					0, PL3_BROWSE_ARTIST,
+					0, artist_browser_plug.id,
 					1, data->tag,
 					2, artist,
 					3, "media-album", 
@@ -497,15 +563,15 @@ void pl3_artist_browser_fill_tree(GtkTreeIter *iter)
 			gtk_tree_store_remove(pl3_tree, &child); 
 		}
 	}
-	g_free(artist);
-	g_free(alb_artist);
+	q_free(artist);
+	q_free(alb_artist);
 }
 
 
 
-void pl3_artist_browser_add_folder()
+static void pl3_artist_browser_add_folder()
 {
-	GtkTreeSelection *selec = gtk_tree_view_get_selection((GtkTreeView *)pl3_cat_tree);
+	GtkTreeSelection *selec = gtk_tree_view_get_selection(playlist3_get_category_tree_view());
 	GtkTreeModel *model = GTK_TREE_MODEL(pl3_tree);
 	GtkTreeIter iter;
 
@@ -519,8 +585,8 @@ void pl3_artist_browser_add_folder()
 		gtk_tree_model_get(model, &iter, PL3_CAT_INT_ID, &artist, PL3_CAT_TITLE,&title, -1);
 		if(!artist || !title)
 		{
-			if(artist)g_free(artist);
-			if(title)g_free(title);
+			if(artist)q_free(artist);
+			if(title)q_free(title);
 			return;
 		}
 		if(strlen(artist) && !g_utf8_collate(artist,title))
@@ -537,7 +603,7 @@ void pl3_artist_browser_add_folder()
 				data = mpd_data_get_next(data);
 			}
 			pl3_push_statusbar_message(message);
-			g_free(message);
+			q_free(message);
 
 		}
 		else
@@ -558,40 +624,38 @@ void pl3_artist_browser_add_folder()
 				data = mpd_data_get_next(data);
 			}
 			pl3_push_statusbar_message(message);
-			g_free(message);
+			q_free(message);
 
 		}
 		/* if there are items in the add list add them to the playlist */
 		mpd_playlist_queue_commit(connection);
-		if(artist)g_free(artist);
-		if(title)g_free(title);
+		if(artist)q_free(artist);
+		if(title)q_free(title);
 	}
 }
 
-void pl3_artist_browser_replace_folder()
+static void pl3_artist_browser_replace_folder()
 {
 	mpd_playlist_clear(connection);
 	pl3_artist_browser_add_folder();
 	mpd_player_play(connection);
 }
 
-void pl3_artist_browser_category_key_press(GdkEventKey *event)
+static void pl3_artist_browser_category_key_press(GtkWidget *tree, GdkEventKey *event, int selected_type)
 {
+	if(selected_type != artist_browser_plug.id) return; 
 	if(event->state&GDK_CONTROL_MASK && event->keyval == GDK_Insert )
 	{
 		pl3_artist_browser_replace_folder();
 	}
 
-
-
-
 	else if (event->keyval == GDK_Insert)
 	{
 		pl3_artist_browser_add_folder();
 	}
 }
 
-void pl3_artist_browser_show_info()
+static void pl3_artist_browser_show_info()
 {
 	GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(pl3_ab_tree));
 	GtkTreeSelection *selection =gtk_tree_view_get_selection (GTK_TREE_VIEW(pl3_ab_tree));
@@ -610,7 +674,6 @@ void pl3_artist_browser_show_info()
 			GtkTreeIter iter;
 			char *path;
 			int type;
-			MpdData *data;
 			gtk_tree_model_get_iter (model, &iter, (GtkTreePath *) list->data);
 			gtk_tree_model_get (GTK_TREE_MODEL(pl3_ab_store), &iter,
 					PL3_AB_FILE, &path,
@@ -618,17 +681,12 @@ void pl3_artist_browser_show_info()
 					-1);
 			if(type == PL3_ENTRY_SONG)
 			{
-				data = mpd_database_find_adv(connection,TRUE,MPD_TAG_ITEM_FILENAME,path,-1);
-				while(data != NULL)
-				{
-					if(data->type == MPD_DATA_TYPE_SONG)
-					{
-						call_id3_window_song(mpd_songDup(data->song));
-					}
-					data = mpd_data_get_next(data);
-				}
+				mpd_Song *song = mpd_database_get_fileinfo(connection, path);
+				if(song)
+					call_id3_window_song(song);
+
 			}
-			if(path)g_free(path);
+			if(path)q_free(path);
 		}
 		while ((list = g_list_previous (list)) && mpd_check_connected(connection));
 		/* free list */
@@ -637,13 +695,15 @@ void pl3_artist_browser_show_info()
 	}
 }
 
-void pl3_artist_browser_row_activated(GtkTreeView *tree, GtkTreePath *tp)
+static void pl3_artist_browser_row_activated(GtkTreeView *tree, GtkTreePath *tp)
 {
 	GtkTreeIter iter;
 	gchar *artist = NULL;
 	gchar *album = NULL;
 	gchar *file = NULL;
 	gint r_type;
+	int playlist_length = mpd_playlist_get_playlist_length(connection);
+
 	gtk_tree_model_get_iter(gtk_tree_view_get_model(tree), &iter, tp);
 	gtk_tree_model_get(gtk_tree_view_get_model(tree), &iter, 
 			PL3_AB_ARTIST, &artist,
@@ -653,7 +713,7 @@ void pl3_artist_browser_row_activated(GtkTreeView *tree, GtkTreePath *tp)
 			-1);
 	if (r_type&(PL3_ENTRY_ARTIST|PL3_ENTRY_ALBUM))
 	{
-		GtkTreeSelection *selec = gtk_tree_view_get_selection((GtkTreeView *)pl3_cat_tree);
+		GtkTreeSelection *selec = gtk_tree_view_get_selection((GtkTreeView *)playlist3_get_category_tree_view());
 		GtkTreeModel *model = GTK_TREE_MODEL(pl3_tree);
 		GtkTreeIter iter;
 
@@ -661,7 +721,7 @@ void pl3_artist_browser_row_activated(GtkTreeView *tree, GtkTreePath *tp)
 		{
 			GtkTreeIter citer;
 			GtkTreePath *path = gtk_tree_model_get_path(model, &iter);
-			gtk_tree_view_expand_row(GTK_TREE_VIEW(pl3_cat_tree), path, FALSE);
+			gtk_tree_view_expand_row(GTK_TREE_VIEW(playlist3_get_category_tree_view()), path, FALSE);
 			gtk_tree_path_free(path);
 			if(gtk_tree_model_iter_children(model, &citer, &iter))
 			{
@@ -672,30 +732,54 @@ void pl3_artist_browser_row_activated(GtkTreeView *tree, GtkTreePath *tp)
 					{
 						gtk_tree_selection_select_iter(selec,&citer);
 						path = gtk_tree_model_get_path(model, &citer);
-						gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(pl3_cat_tree),
+						gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(playlist3_get_category_tree_view()),
 								path,NULL,TRUE,0.5,0);
 						gtk_tree_path_free(path);
-						g_free(name);
+						q_free(name);
 						break;
 					}
-					g_free(name);
+					q_free(name);
 				}while(gtk_tree_model_iter_next(model, &citer));
 			}
 		}
 	}
+	else if(r_type&PL3_ENTRY_DIR_UP)
+	{
+		GtkTreeSelection *selec = gtk_tree_view_get_selection((GtkTreeView *)playlist3_get_category_tree_view());
+		GtkTreeModel *model = GTK_TREE_MODEL(pl3_tree);
+		GtkTreeIter iter,parent;
+
+		if(gtk_tree_selection_get_selected(selec,&model, &iter))
+		{
+			gtk_tree_model_iter_parent(model, &parent, &iter);
+			gtk_tree_selection_select_iter(selec, &parent);
+		}
+	}
 	else
 	{
-		pl3_push_statusbar_message("Added a song");
-		mpd_playlist_queue_add(connection, file);
+		pl3_push_statusbar_message(_("Added a song"));
+		if(mpd_server_check_command_allowed(connection, "addid") == MPD_SERVER_COMMAND_ALLOWED){
+			int songid = mpd_playlist_add_get_id(connection, file);
+			if(songid >= 0) {
+				mpd_player_play_id(connection, songid);
+			}
+		} else{
+			mpd_playlist_add(connection, file);
+			if(playlist_length == 0)
+			{
+				mpd_player_play(connection);
+			}
+		}
 	}
 
-	mpd_playlist_queue_commit(connection);
-	if(file)g_free(file);
-	if(artist) g_free(artist);
-	if(album) g_free(album);
+
+	if(file)q_free(file);
+	if(artist) q_free(artist);
+	if(album) q_free(album);
+
 }
 
-void pl3_artist_browser_category_selection_changed(GtkTreeView *tree,GtkTreeIter *iter)
+static void pl3_artist_browser_category_selection_changed(GtkWidget *tree,GtkTreeIter *iter)
 {
 	long unsigned time= 0;
 	gchar *string;
@@ -703,12 +787,10 @@ void pl3_artist_browser_category_selection_changed(GtkTreeView *tree,GtkTreeIter
 	time = pl3_artist_browser_view_folder(iter);
 	string = format_time(time);
 	pl3_push_rsb_message(string);
-	g_free(string);
-	/* store the tree */
-	pl3_cat_tree= GTK_WIDGET(tree);
+	q_free(string);
 }
 
-void pl3_artist_browser_selected(GtkWidget *container)
+static void pl3_artist_browser_selected(GtkWidget *container)
 {
 	if(pl3_ab_tree == NULL)
 	{
@@ -716,14 +798,15 @@ void pl3_artist_browser_selected(GtkWidget *container)
 	}
 
 	gtk_container_add(GTK_CONTAINER(container), pl3_ab_vbox);
+	gtk_widget_grab_focus(pl3_ab_tree);
 	gtk_widget_show(pl3_ab_vbox);
 }
-void pl3_artist_browser_unselected(GtkWidget *container)
+static void pl3_artist_browser_unselected(GtkWidget *container)
 {
 	gtk_container_remove(GTK_CONTAINER(container),pl3_ab_vbox);
 }
 
-void pl3_artist_browser_button_release_event(GtkWidget *but, GdkEventButton *event)
+static void pl3_artist_browser_button_release_event(GtkWidget *but, GdkEventButton *event)
 {
 	if(event->button != 3) return;
 	GtkWidget *item;
@@ -774,20 +857,22 @@ void pl3_artist_browser_button_release_event(GtkWidget *but, GdkEventButton *eve
 	return;
 }
 
-void pl3_artist_browser_replace_selected()
+static void pl3_artist_browser_replace_selected()
 {
 	mpd_playlist_clear(connection);
 	pl3_artist_browser_add_selected();
 	mpd_player_play(connection);
 }
-void pl3_artist_browser_add_selected()
+
+static void pl3_artist_browser_add_selected()
 {
 	GtkTreeIter iter;
 	GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(pl3_ab_tree));
 	GtkTreeModel *model = GTK_TREE_MODEL (pl3_ab_store);
 	GList *rows = gtk_tree_selection_get_selected_rows (selection, &model);
 	int songs=0;
-	gchar *message;
+	int artists = 0;
+	int albums = 0;
 	if(rows != NULL)
 	{
 		gchar *artist,*album,*file;
@@ -816,10 +901,10 @@ void pl3_artist_browser_add_selected()
 					if(data->type == MPD_DATA_TYPE_SONG)
 					{
 						mpd_playlist_queue_add(connection, data->song->file);
-						songs++;
 					}
 					data = mpd_data_get_next(data);
 				}
+				artists++;
 			}
 			else if (type&PL3_ENTRY_ALBUM)
 			{
@@ -831,33 +916,46 @@ void pl3_artist_browser_add_selected()
 					{
 						if (!g_utf8_collate (data->song->artist, artist))
 						{
-							songs++;
+
 							mpd_playlist_queue_add(connection,data->song->file);
 						}
 					}
 					data = mpd_data_get_next(data);
 				}
+				albums++;
 			}
-			if(artist)g_free(artist);
-			if(album)g_free(album);
-			if(file)g_free(file);
+			if(artist)q_free(artist);
+			if(album)q_free(album);
+			if(file)q_free(file);
 
 		}while((node = g_list_next(node)) != NULL);
 	}
 	/* if there are items in the add list add them to the playlist */
 	mpd_playlist_queue_commit(connection);
-	if(songs != 0)
+	if(songs != 0 || artists || albums)
 	{
-		message = g_strdup_printf("Added %i song%s", songs, (songs != 1)? "s":"");
-		pl3_push_statusbar_message(message);
-		g_free(message);
+		GString *str = g_string_new("Added: ");
+		if(songs)
+		{
+			g_string_append_printf(str, "%i %s", songs, (songs > 1)?_("songs"):_("song"));
+		}		
+		if(artists)
+		{
+			g_string_append_printf(str, "%i %s", artists, (artists > 1)?_("artists"):_("artist"));
+		}		
+		if(albums)
+		{
+			g_string_append_printf(str, "%i %s", albums, (albums > 1)?_("albums"):_("album"));
+		}		
+		pl3_push_statusbar_message(str->str);
+		g_string_free(str, TRUE);
 	}
 
 	g_list_foreach (rows, (GFunc) gtk_tree_path_free, NULL);
 	g_list_free (rows);
 }
 
-int pl3_artist_browser_playlist_key_press(GtkWidget *tree, GdkEventKey *event)
+static int pl3_artist_browser_playlist_key_press(GtkWidget *tree, GdkEventKey *event)
 {
 	if(event->state&GDK_CONTROL_MASK && event->keyval == GDK_Insert)
 	{
@@ -871,10 +969,21 @@ int pl3_artist_browser_playlist_key_press(GtkWidget *tree, GdkEventKey *event)
 	{
 		pl3_artist_browser_add_selected();
 	}
-	else if(event->keyval == GDK_i)
+	else if(event->keyval == GDK_i && event->state&GDK_MOD1_MASK)
 	{
 		pl3_artist_browser_show_info();
 	}
+	else if((event->state&(GDK_CONTROL_MASK|GDK_MOD1_MASK)) == 0 && 
+		((event->keyval >= GDK_space && event->keyval <= GDK_z)))
+	{
+		char data[2];
+		data[0] = (char)gdk_keyval_to_unicode(event->keyval);
+		data[1] = '\0';
+		treesearch_start(TREESEARCH(pl3_ab_tree_search));
+		gtk_entry_set_text(GTK_ENTRY(TREESEARCH(pl3_ab_tree_search)->entry),data);
+		gtk_editable_set_position(GTK_EDITABLE(TREESEARCH(pl3_ab_tree_search)->entry),1);
+		return TRUE;
+	}
 	else
 	{
 		return pl3_window_key_press_event(tree,event);
@@ -882,10 +991,24 @@ int pl3_artist_browser_playlist_key_press(GtkWidget *tree, GdkEventKey *event)
 	return TRUE;
 }
 
-int pl3_artist_browser_cat_popup(GtkWidget *menu, int type,GtkTreeView *tree, GdkEventButton *event)
+static int pl3_artist_browser_cat_popup(GtkWidget *menu, int type,GtkWidget *tree, GdkEventButton *event)
 {
-	if(type == PL3_BROWSE_ARTIST)
+	if(type == artist_browser_plug.id)
 	{
+		GtkTreeSelection *selec = gtk_tree_view_get_selection((GtkTreeView *)glade_xml_get_widget (pl3_xml, "cat_tree"));
+		GtkTreePath *path = NULL;
+		GtkTreeModel *model = NULL;
+		GtkTreeIter iter;
+		/* check if the right item */
+		if(!gtk_tree_selection_get_selected(selec, &model, &iter))
+			return 0;
+		path = gtk_tree_model_get_path(model, &iter);
+		if(gtk_tree_path_get_depth(path) == 1){
+			gtk_tree_path_free(path);
+			return 0;
+		}
+		gtk_tree_path_free(path);
+
 		/* here we have:  Add. Replace*/
 		GtkWidget *item;
 		/* add the add widget */
@@ -907,7 +1030,7 @@ int pl3_artist_browser_cat_popup(GtkWidget *menu, int type,GtkTreeView *tree, Gd
 
 
 
-void pl3_artist_browser_disconnect()
+static void pl3_artist_browser_disconnect()
 {
 	if(pl3_ab_tree_ref) {
 		GtkTreeIter iter;
@@ -927,3 +1050,87 @@ void pl3_artist_browser_disconnect()
 		if(pl3_ab_store) gtk_list_store_clear(pl3_ab_store);
 	}
 }
+
+
+
+static void pl3_artist_browser_activate()
+{
+	GtkTreeSelection *selec = gtk_tree_view_get_selection((GtkTreeView *)
+			glade_xml_get_widget (pl3_xml, "cat_tree"));
+
+	/**
+	 * Fix this to be nnot static
+	 */	
+	GtkTreePath *path = gtk_tree_row_reference_get_path(pl3_ab_tree_ref); 
+	if(path)
+	{
+		gtk_tree_selection_select_path(selec, path);
+		gtk_tree_path_free(path);
+	}
+}
+
+
+static int pl3_artist_browser_add_go_menu(GtkWidget *menu)
+{
+	GtkWidget *item = NULL;
+
+	item = gtk_image_menu_item_new_with_label(_("Artist Browser"));
+	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), 
+			gtk_image_new_from_stock("media-artist", GTK_ICON_SIZE_MENU));
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+	g_signal_connect(G_OBJECT(item), "activate", 
+			G_CALLBACK(pl3_artist_browser_activate), NULL);
+
+	return 1;
+}
+
+static void pl3_artist_browser_connection_changed(MpdObj *mi, int connect, gpointer data)
+{
+	if(!connect)
+	{
+		pl3_artist_browser_disconnect();
+	}
+}
+static int pl3_artist_browser_key_press_event(GtkWidget *mw, GdkEventKey *event, int type)
+{
+	if (event->keyval == GDK_F4)
+	{
+		pl3_artist_browser_activate();
+		return TRUE;
+	}                                           	
+
+	return FALSE;
+}
+static void pl3_artist_browser_status_changed(MpdObj *mi,ChangedStatusType what, void *data)
+{
+	if(what&MPD_CST_DATABASE)
+	{
+		pl3_artist_browser_reupdate();
+	}
+}	
+
+static void pl3_artist_browser_reupdate(void)
+{
+	pl3_artist_browser_disconnect();
+	
+	if(pl3_ab_tree_ref && pl3_cat_get_selected_browser() == artist_browser_plug.id){
+		GtkTreePath *path = gtk_tree_row_reference_get_path(pl3_ab_tree_ref);
+		if(path)
+		{
+			GtkTreeIter parent;
+			if(gtk_tree_model_get_iter(GTK_TREE_MODEL(pl3_tree), &parent, path))
+			{
+				long unsigned time= 0;
+				gchar *string;
+
+				time = pl3_artist_browser_view_folder(&parent);
+				string = format_time(time);
+				gtk_statusbar_push(GTK_STATUSBAR(glade_xml_get_widget(pl3_xml, "statusbar2")),0, string);
+				q_free(string);
+			}
+			gtk_tree_path_free(path);
+		}
+	}
+}
+
+
