@@ -1,5 +1,5 @@
 /*
- *Copyright (C) 2004-2005 Qball Cow <Qball@qballcow.nl>
+ *Copyright (C) 2004-2007 Qball Cow <qball@sarine.nl>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,17 +34,67 @@
 #include "config1.h"
 #include "TreeSearchWidget.h"
 
-void pl3_file_browser_button_release_event(GtkWidget *but, GdkEventButton *event);
-void pl3_file_browser_row_activated(GtkTreeView *tree, GtkTreePath *tp);
-void pl3_file_browser_add_selected();
-void pl3_file_browser_replace_selected();
-int pl3_file_browser_playlist_key_press(GtkWidget *tree,GdkEventKey *event);
-void pl3_file_browser_show_info();
-long unsigned pl3_file_browser_view_folder(GtkTreeIter *iter_cat);
-void pl3_file_browser_update_folder();
-void pl3_file_browser_add_folder();
-void pl3_file_browser_delete_playlist(GtkMenuItem *bt);
-void pl3_file_browser_reupdate_folder(GtkTreeIter *parent, char *path);
+static void pl3_file_browser_add(GtkWidget *cat_tree);
+static void pl3_file_browser_unselected(GtkWidget *container);
+static void pl3_file_browser_selected(GtkWidget *container);
+static void pl3_file_browser_cat_sel_changed(GtkWidget *tree,GtkTreeIter *iter);
+static void pl3_file_browser_fill_tree(GtkWidget *tree,GtkTreeIter *iter);
+static int pl3_file_browser_cat_popup(GtkWidget *menu, int type,GtkWidget *tree, GdkEventButton *event);
+static void pl3_file_browser_cat_key_press(GtkWidget *tree, GdkEventKey *event, int selected_type);
+static void pl3_file_browser_delete_playlist_from_right(GtkMenuItem *bt);
+/* testing */
+static void pl3_file_browser_reupdate(void);
+
+static int pl3_file_browser_add_go_menu(GtkWidget *menu);
+static void pl3_file_browser_activate(void);
+static void pl3_file_browser_button_release_event(GtkWidget *but, GdkEventButton *event);
+static void pl3_file_browser_row_activated(GtkTreeView *tree, GtkTreePath *tp);
+static void pl3_file_browser_add_selected(void);
+static void pl3_file_browser_replace_selected(void);
+static int pl3_file_browser_playlist_key_press(GtkWidget *tree,GdkEventKey *event);
+static void pl3_file_browser_show_info(void);
+static long unsigned pl3_file_browser_view_folder(GtkTreeIter *iter_cat);
+static void pl3_file_browser_update_folder(void);
+static void pl3_file_browser_add_folder(void);
+static void pl3_file_browser_delete_playlist(GtkMenuItem *bt);
+static void pl3_file_browser_connection_changed(MpdObj *mi, int connect, gpointer data);
+static void pl3_file_browser_status_changed(MpdObj *mi,ChangedStatusType what, void *data);
+static int pl3_file_browser_key_press_event(GtkWidget *mw, GdkEventKey *event, int type);
+static void pl3_file_browser_disconnect(void);
+
+guint pl3_fb_filling_source = 0;
+/**
+ * Plugin structure
+ */
+gmpcPlBrowserPlugin file_browser_gbp = {
+	pl3_file_browser_add,
+	pl3_file_browser_selected,
+	pl3_file_browser_unselected,
+	pl3_file_browser_cat_sel_changed,
+	pl3_file_browser_fill_tree,
+	pl3_file_browser_cat_popup,
+	pl3_file_browser_cat_key_press,
+	pl3_file_browser_add_go_menu,
+	pl3_file_browser_key_press_event
+};
+
+gmpcPlugin file_browser_plug = {
+	"File Browser",
+	{1,1,1},
+	GMPC_PLUGIN_PL_BROWSER,
+	0,
+	NULL,			                /* path*/
+	NULL,			                /* init */
+        NULL,                                   /* destroy */
+	&file_browser_gbp,		        /* Browser */
+	pl3_file_browser_status_changed,        /* status changed */
+	pl3_file_browser_connection_changed, 	/* connection changed */
+	NULL,		                        /* Preferences */
+	NULL,			                /* MetaData */
+	NULL,                                   /* get enable */
+	NULL                                    /* set enable */
+};
+
 
 enum{
 	PL3_FB_PATH,
@@ -63,7 +113,7 @@ GtkListStore *pl3_fb_store = NULL;
 GtkWidget *pl3_fb_vbox = NULL;
 GtkWidget *pl3_fb_tree_search = NULL;
 
-int pl3_file_browser_button_press_event(GtkTreeView *tree, GdkEventButton *event)
+static int pl3_file_browser_button_press_event(GtkTreeView *tree, GdkEventButton *event)
 {
 	GtkTreeSelection *sel = gtk_tree_view_get_selection(tree);
 	if(event->button != 3 || gtk_tree_selection_count_selected_rows(sel) < 2|| !mpd_check_connected(connection))
@@ -88,24 +138,28 @@ static void pl3_file_browser_search_activate()
 }
 
 
-void pl3_file_browser_init()
+static void pl3_file_browser_init()
 {
 	GtkWidget *pl3_fb_sw = NULL;
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column = NULL;
 	GValue value;
 	pl3_fb_store = gtk_list_store_new (PL3_FB_ROWS,
-			GTK_TYPE_STRING, /* path to file */
-			GTK_TYPE_INT,	/* type, FILE/PLAYLIST/FOLDER  */
-			GTK_TYPE_STRING,	/* title to display */
-			GTK_TYPE_STRING); /* icon type */
-
+			G_TYPE_STRING, /* path to file */
+			G_TYPE_INT,	/* type, FILE/PLAYLIST/FOLDER  */
+			G_TYPE_STRING,	/* title to display */
+			G_TYPE_STRING); /* icon type */
 
+	/* set up the tree */
+	pl3_fb_tree= gtk_tree_view_new_with_model(GTK_TREE_MODEL(pl3_fb_store));
 
 	renderer = gtk_cell_renderer_pixbuf_new ();
 
 	column = gtk_tree_view_column_new ();
-	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	gtk_tree_view_column_set_fixed_width(column, 300);
+	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
+
+	gtk_tree_view_set_fixed_height_mode(GTK_TREE_VIEW(pl3_fb_tree), TRUE);
 	gtk_tree_view_column_pack_start (column, renderer, FALSE);
 	gtk_tree_view_column_set_attributes (column,renderer,"stock-id", PL3_FB_ICON,NULL);
 	memset(&value, 0, sizeof(value));
@@ -119,8 +173,8 @@ void pl3_file_browser_init()
 	gtk_tree_view_column_set_attributes (column,renderer,"text", PL3_FB_TITLE, NULL);
 
 
-	/* set up the tree */
-	pl3_fb_tree= gtk_tree_view_new_with_model(GTK_TREE_MODEL(pl3_fb_store));
+
+
 	/* insert the column in the tree */
 	gtk_tree_view_append_column (GTK_TREE_VIEW (pl3_fb_tree), column);
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(pl3_fb_tree), FALSE);
@@ -153,9 +207,9 @@ void pl3_file_browser_init()
 	g_object_ref(G_OBJECT(pl3_fb_vbox));
 }
 
-void pl3_file_browser_add_folder()
+static void pl3_file_browser_add_folder()
 {
-	GtkTreeSelection *selec = gtk_tree_view_get_selection((GtkTreeView *)glade_xml_get_widget (pl3_xml, "cat_tree"));
+	GtkTreeSelection *selec = gtk_tree_view_get_selection(playlist3_get_category_tree_view());
 	GtkTreeModel *model = GTK_TREE_MODEL(pl3_tree);
 	GtkTreeIter iter;
 
@@ -165,19 +219,27 @@ void pl3_file_browser_add_folder()
 	}
 	if(gtk_tree_selection_get_selected(selec,&model, &iter))
 	{
-		char *path;
+		char *path, *icon;
 		char *message = NULL;
-		gtk_tree_model_get(model, &iter, PL3_CAT_INT_ID, &path, -1);
-		message = g_strdup_printf("Added folder '%s' recursively", path);
+		gtk_tree_model_get(model, &iter, PL3_CAT_INT_ID, &path,3, &icon, -1);
+	
+	
+		message = g_strdup_printf(_("Added folder '%s' recursively"), path);
 		pl3_push_statusbar_message(message);
-		g_free(message);
-		mpd_playlist_queue_add(connection, path);
+		q_free(message);
+		if(strcmp("media-playlist", icon)) {                                 		
+			mpd_playlist_queue_add(connection, path);
+		}
+		else {
+			mpd_playlist_queue_load(connection, path);
+		}
 		mpd_playlist_queue_commit(connection);
-		g_free(path);
+		q_free(path);
+		q_free(icon);
 	}
 }
 
-void pl3_file_browser_update_folder()
+static void pl3_file_browser_update_folder()
 {
 	GtkTreeSelection *selec = gtk_tree_view_get_selection((GtkTreeView *)glade_xml_get_widget (pl3_xml, "cat_tree"));
 	GtkTreeModel *model = GTK_TREE_MODEL(pl3_tree);
@@ -191,13 +253,16 @@ void pl3_file_browser_update_folder()
 	{
 		char *path;
 		gtk_tree_model_get(model, &iter, PL3_CAT_INT_ID, &path, -1);
-		mpd_database_update_dir(connection, path);
-		g_free(path);
+		if(path)
+		{
+			mpd_database_update_dir(connection, path);
+			q_free(path);
+		}
 	}
 
 }
 
-void pl3_file_browser_replace_folder()
+static void pl3_file_browser_replace_folder()
 {
 	mpd_playlist_clear(connection);
 	pl3_file_browser_add_folder();	
@@ -206,14 +271,14 @@ void pl3_file_browser_replace_folder()
 
 
 /* add's the toplevel entry for the file browser, it also add's a fantom child */
-void pl3_file_browser_add()
+static void pl3_file_browser_add(GtkWidget *cat_tree)
 {
 	GtkTreeIter iter,child;
 	GtkTreePath *path;
 	gtk_tree_store_append(pl3_tree, &iter, NULL);
 	gtk_tree_store_set(pl3_tree, &iter, 
-			PL3_CAT_TYPE, PL3_BROWSE_FILE,
-			PL3_CAT_TITLE, "Browse Filesystem",
+			PL3_CAT_TYPE, file_browser_plug.id,
+			PL3_CAT_TITLE, _("Browse Filesystem"),
 			PL3_CAT_INT_ID, "/",
 			PL3_CAT_ICON_ID, "gtk-open",
 			PL3_CAT_PROC, FALSE,
@@ -233,148 +298,75 @@ void pl3_file_browser_add()
 		gtk_tree_path_free(path);
 	}
 }
-void pl3_file_browser_reupdate_folder(GtkTreeIter *parent, char *path)
-{
-	MpdData *data = mpd_database_get_directory(connection,path);
-	if(data == NULL)
-	{
-		/* clear the tree */
-		gtk_tree_store_remove(pl3_tree, parent);
-	}
-	else
-	{
-		GtkTreeIter iter;
-		int valid = 0;
-		valid = gtk_tree_model_iter_children(GTK_TREE_MODEL(pl3_tree), &iter, parent);
-		while(valid){
-			char *ppath = NULL;
-			int found = 0;
-			gtk_tree_model_get(GTK_TREE_MODEL(pl3_tree), &iter, 2, &ppath,-1);
-			if(ppath && data)
-			{
-				MpdData *list = mpd_data_get_first(data);
-				do{
-					if(list->type ==  MPD_DATA_TYPE_DIRECTORY)
-					{
-						if(!strcmp(ppath, list->directory)){
-							data = list;
-						       	found = TRUE;
-						}
-					}
-				}while(!mpd_data_is_last(list) && (list = mpd_data_get_next(list)));
-			}
-			if(!found)
-			{
-				/** removes the row, and set iter to the next one */
-				valid = gtk_tree_store_remove(pl3_tree, &iter);
-			}
-			else
-			{
 
-				pl3_file_browser_reupdate_folder(&iter,ppath);
-				valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(pl3_tree), &iter);
-				data = mpd_data_delete_item(data);
-			}
-			g_free(ppath);
-		}
-		if(data)
-		{
-			GtkTreeIter child,child2;
-			data = mpd_data_get_first(data);
-			while (data != NULL)
-			{
-				if (data->type == MPD_DATA_TYPE_DIRECTORY)
-				{
-					gchar *basename =
-						g_path_get_basename (data->directory);
-					gtk_tree_store_append (pl3_tree, &child, parent);
-					gtk_tree_store_set (pl3_tree, &child,
-							0, PL3_BROWSE_FILE,
-							1, basename,
-							2, data->directory,
-							3, "gtk-open",
-							4, FALSE,
-							PL3_CAT_ICON_SIZE,1,
-							-1);
-					gtk_tree_store_append(pl3_tree, &child2, &child);
-
-					g_free (basename);
-				}
-				data = mpd_data_get_next(data);
-			}
-		}
-	}
-}
-void pl3_file_browser_reupdate()
+static void pl3_file_browser_reupdate()
 {
-	if(pl3_fb_tree_ref){
+	pl3_file_browser_disconnect();
+	
+	if(pl3_fb_tree_ref && pl3_cat_get_selected_browser() == file_browser_plug.id){
+/*
 		GtkTreePath *path = gtk_tree_row_reference_get_path(pl3_fb_tree_ref);
 		if(path)
 		{
 			GtkTreeIter parent;
-			MpdData *data = mpd_database_get_directory(connection, "/");
-			gtk_tree_model_get_iter(GTK_TREE_MODEL(pl3_tree), &parent, path);
-			if(data == NULL)
+			if(gtk_tree_model_get_iter(GTK_TREE_MODEL(pl3_tree), &parent, path))
 			{
-				GtkTreeIter iter;
-				int valid = gtk_tree_model_iter_children(GTK_TREE_MODEL(pl3_tree), &iter, &parent);
-				debug_printf(DEBUG_INFO,"clearing complete tree\n");
-				while(valid){
-					valid = gtk_tree_store_remove(pl3_tree,&iter);
-				}
-			}
-			else
-			{
-				pl3_file_browser_reupdate_folder(&parent, "/");
+				long unsigned time= 0;
+				gchar *string;
 
+				time = pl3_file_browser_view_folder(&parent);
+				string = format_time(time);
+				gtk_statusbar_push(GTK_STATUSBAR(glade_xml_get_widget(pl3_xml, "statusbar2")),0, string);
+				q_free(string);
 			}
-			/* update right view */
-			if(pl3_cat_get_selected_browser() == PL3_BROWSE_FILE)
-			{
-				GtkTreeSelection *selec = gtk_tree_view_get_selection((GtkTreeView *)
-						glade_xml_get_widget (pl3_xml, "cat_tree"));
-				GtkTreeModel *model = GTK_TREE_MODEL(pl3_tree);
-
-				gtk_tree_selection_get_selected(selec,&model, &parent);
-			}
-			pl3_file_browser_view_folder(&parent);
-			mpd_data_free(data);
 			gtk_tree_path_free(path);
 		}
+*/
+		GtkTreeIter iter;
+		
+		GtkTreeModel *model = gtk_tree_row_reference_get_model(pl3_fb_tree_ref);
+		GtkTreePath *path = gtk_tree_row_reference_get_path(pl3_fb_tree_ref);
+		if(gtk_tree_model_get_iter(model, &iter, path))
+		{
+			GtkTreeSelection *sel= gtk_tree_view_get_selection(playlist3_get_category_tree_view());
+			gtk_tree_selection_select_iter(sel, &iter);
+
+/*			if(pl3_cat_get_selected_browser() == file_browser_plug.id)
+			{
+*/				pl3_file_browser_view_folder(&iter);
+/*			}
+*/		}
+		gtk_tree_path_free(path);
+/*		pl3_file_browser_activate();*/
 	}
 }
 
-long unsigned pl3_file_browser_view_folder(GtkTreeIter *iter_cat)
+typedef struct {
+	MpdData *data;
+	int sub_folder;
+	int time;
+	int support_playlist;
+	GtkTreeIter *iter_cat;
+}pl3_fb_lf_pb;
+
+static void pl3_fb_lazy_fill_destroy(pl3_fb_lf_pb *pb)
+{
+	gtk_tree_view_set_model(GTK_TREE_VIEW(pl3_fb_tree), GTK_TREE_MODEL(pl3_fb_store));
+	if(pb->data)
+	{
+		mpd_data_free(pb->data);
+	}
+	q_free(pb);
+	pb = NULL;
+	/* remove the context id, so we don't try to remove it another time. */
+	pl3_fb_filling_source = 0;
+}
+
+static int pl3_fb_lazy_fill ( pl3_fb_lf_pb *pd)
 {
-	MpdData* data =NULL;
-	char *path;
-	int sub_folder = 0;
 	GtkTreeIter iter;
-	long  unsigned time=0;
-
-	if(pl3_fb_store == NULL) return 0;
-	gtk_list_store_clear(pl3_fb_store);
-	/* check the connection state and when its valid proceed */
-	if (!mpd_check_connected(connection))
-	{
-		return 0;
-	}
-
-	gtk_tree_model_get(GTK_TREE_MODEL(pl3_tree), iter_cat, 2 , &path, -1);
-	if(strcmp(path,"/"))
-	{
-		gtk_list_store_append (pl3_fb_store, &iter);
-		gtk_list_store_set (pl3_fb_store, &iter,
-				PL3_FB_PATH,path, 
-				PL3_FB_TYPE, PL3_ENTRY_DIR_UP,
-				PL3_FB_TITLE, "..",
-				PL3_FB_ICON, "gtk-open",
-				-1);
-	}
-
-
-	data = mpd_database_get_directory(connection, path);
-	while (data != NULL)
+	MpdData *data = pd->data;
+	if (data != NULL)
 	{
 		if (data->type == MPD_DATA_TYPE_DIRECTORY)
 		{
@@ -386,8 +378,8 @@ long unsigned pl3_file_browser_view_folder(GtkTreeIter *iter_cat)
 					PL3_FB_TITLE, basename,
 					PL3_FB_ICON, "gtk-open",
 					-1);
-			g_free(basename);
-			sub_folder++;
+			q_free(basename);
+			(pd->sub_folder)++;
 		}
 		else if (data->type == MPD_DATA_TYPE_SONG)
 		{
@@ -397,7 +389,7 @@ long unsigned pl3_file_browser_view_folder(GtkTreeIter *iter_cat)
 			cfg_free_string(markdata);
 			if(data->song->time != MPD_SONG_NO_TIME)
 			{
-				time += data->song->time;
+				pd->time += data->song->time;
 			}
 
 			gtk_list_store_append (pl3_fb_store, &iter);
@@ -420,31 +412,85 @@ long unsigned pl3_file_browser_view_folder(GtkTreeIter *iter_cat)
 					PL3_FB_TITLE, basename,
 					PL3_FB_ICON, "media-playlist",
 					-1);
-			g_free (basename);
+			q_free (basename);
+			if(pd->support_playlist) (pd->sub_folder)++;
 		}
-		data = mpd_data_get_next(data);
+		pd->data = mpd_data_get_next(data);
+		if(pd->data == NULL) {
+			return FALSE;
+		}
+		return TRUE;
 	}
-	/* remove the fantom child if there are no subfolders anyway. */
-	if(!sub_folder)
+	return FALSE;
+} 
+
+
+static long unsigned pl3_file_browser_view_folder(GtkTreeIter *iter_cat)
+{
+	pl3_fb_lf_pb *pb = NULL;
+		MpdData* data =NULL;
+	char *path = NULL, *icon = NULL;
+	GtkTreeIter iter;
+	long  unsigned time=0;
+	int support_playlist = (mpd_server_check_command_allowed(connection, "listplaylistinfo") == MPD_SERVER_COMMAND_ALLOWED);
+
+	if(pl3_fb_store == NULL) return 0;
+	gtk_list_store_clear(pl3_fb_store);
+	/* check the connection state and when its valid proceed */
+	if (!mpd_check_connected(connection))
 	{
-		if(gtk_tree_model_iter_children(GTK_TREE_MODEL(pl3_tree), &iter, iter_cat))
-		{
-			gtk_tree_store_remove(pl3_tree, &iter);
-		}
+		return 0;
 	}
-	g_free(path);
+	gtk_tree_model_get(GTK_TREE_MODEL(pl3_tree), iter_cat, 2 , &path,3,&icon, -1);
+	if(strcmp("media-playlist",icon))
+	{
+		debug_printf(DEBUG_INFO,"View Folder\n");
+		if(strcmp(path,"/"))
+		{
+			gtk_list_store_append (pl3_fb_store, &iter);
+			gtk_list_store_set (pl3_fb_store, &iter,
+					PL3_FB_PATH,path, 
+					PL3_FB_TYPE, PL3_ENTRY_DIR_UP,
+					PL3_FB_TITLE, "..",
+					PL3_FB_ICON, "gtk-open",
+					-1);
+		}
+
+
+		data = mpd_database_get_directory(connection, path);
+	}
+	else{
+		debug_printf(DEBUG_INFO,"View Playlist\n");
+		data = mpd_database_get_playlist_content(connection, path);
+	}
+
+	q_free(path);
+	q_free(icon);
+	pb = g_malloc0(sizeof(*pb));
+	pb->data= data;
+	pb->sub_folder = 0;
+	pb->time = 0;
+	pb->support_playlist = support_playlist;
+	pb->iter_cat = iter_cat;
+
+	gtk_tree_view_set_model(GTK_TREE_VIEW(pl3_fb_tree), NULL);
+	if(pl3_fb_filling_source){
+		g_source_remove(pl3_fb_filling_source);
+	}
+	pl3_fb_filling_source = g_idle_add_full(G_PRIORITY_DEFAULT_IDLE,(GSourceFunc)pl3_fb_lazy_fill,pb, (GDestroyNotify)pl3_fb_lazy_fill_destroy);
 	return time;
 }
 
 
-void pl3_file_browser_fill_tree(GtkTreeIter *iter)
+static void pl3_file_browser_fill_tree(GtkWidget *tree,GtkTreeIter *iter)
 {
 	char *path;
+	int support_playlist = (mpd_server_check_command_allowed(connection, "listPlaylistInfo") == MPD_SERVER_COMMAND_ALLOWED);
 	MpdData *data = NULL;
 	GtkTreeIter child,child2;
 	gtk_tree_model_get(GTK_TREE_MODEL(pl3_tree),iter, 2, &path, -1);
 	gtk_tree_store_set(pl3_tree, iter, 4, TRUE, -1);
-
+	
 	data = mpd_database_get_directory(connection, path);
 	while (data != NULL)
 	{
@@ -454,7 +500,7 @@ void pl3_file_browser_fill_tree(GtkTreeIter *iter)
 				g_path_get_basename (data->directory);
 			gtk_tree_store_append (pl3_tree, &child, iter);
 			gtk_tree_store_set (pl3_tree, &child,
-					0, PL3_BROWSE_FILE,
+					0, file_browser_plug.id,
 					1, basename,
 					2, data->directory,
 					3, "gtk-open",
@@ -463,7 +509,21 @@ void pl3_file_browser_fill_tree(GtkTreeIter *iter)
 					-1);
 			gtk_tree_store_append(pl3_tree, &child2, &child);
 
-			g_free (basename);
+			q_free (basename);
+		}
+		else if(support_playlist && data->type == MPD_DATA_TYPE_PLAYLIST)
+		{
+			gchar *basename = g_path_get_basename(data->playlist);
+			gtk_tree_store_append (pl3_tree, &child, iter);
+			gtk_tree_store_set (pl3_tree, &child,
+					0, file_browser_plug.id,
+					1, basename,
+					2, data->playlist,
+					3, "media-playlist",
+					4, TRUE,
+					PL3_CAT_ICON_SIZE,1,
+					-1);
+			q_free(basename);
 		}
 		data = mpd_data_get_next(data);
 	}
@@ -471,14 +531,14 @@ void pl3_file_browser_fill_tree(GtkTreeIter *iter)
 	{
 		gtk_tree_store_remove(pl3_tree, &child);
 	}
-	g_free(path);
+	q_free(path);
 }
 
 
 
-int pl3_file_browser_cat_popup(GtkWidget *menu, int type,GtkTreeView *tree, GdkEventButton *event)
+static int pl3_file_browser_cat_popup(GtkWidget *menu, int type,GtkWidget *tree, GdkEventButton *event)
 {
-	if(type == PL3_BROWSE_FILE)
+	if(type == file_browser_plug.id)
 	{
 		/* here we have:  Add. Replace, (update?)*/
 		GtkWidget *item;
@@ -488,27 +548,48 @@ int pl3_file_browser_cat_popup(GtkWidget *menu, int type,GtkTreeView *tree, GdkE
 		g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(pl3_file_browser_add_folder), NULL);
 
 		/* add the replace widget */
-		item = gtk_image_menu_item_new_with_label("Replace");
+		item = gtk_image_menu_item_new_with_label(_("Replace"));
 		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
 				gtk_image_new_from_stock(GTK_STOCK_REDO, GTK_ICON_SIZE_MENU));
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 		g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(pl3_file_browser_replace_folder), NULL);
 
-		/* add the update widget */
-		item = gtk_image_menu_item_new_with_label("Update");
-		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
-				gtk_image_new_from_stock(GTK_STOCK_REFRESH, GTK_ICON_SIZE_MENU));
-		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-		g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(pl3_file_browser_update_folder), NULL);	
-
+		{
+			GtkTreeView *tree = playlist3_get_category_tree_view();
+			GtkTreeModel *model = (GtkTreeModel *) playlist3_get_category_tree_store();
+			GtkTreeSelection *selection  = gtk_tree_view_get_selection(tree);
+			GtkTreeIter iter;
+			if(gtk_tree_selection_get_selected(selection, &model, &iter))
+			{
+				char *icon = NULL;
+				gtk_tree_model_get(model, &iter,3, &icon, -1);
+				if(!strcmp("media-playlist", icon))
+				{
+					item = gtk_image_menu_item_new_from_stock(GTK_STOCK_DELETE, NULL);
+					gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+					g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(pl3_file_browser_delete_playlist_from_right), NULL);
+				}
+				else
+				{
+					/* add the update widget */
+					item = gtk_image_menu_item_new_with_label(_("Update"));
+					gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
+							gtk_image_new_from_stock(GTK_STOCK_REFRESH, GTK_ICON_SIZE_MENU));
+					gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+					g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(pl3_file_browser_update_folder), NULL);	
+				}
+				q_free(icon);
+			}
+		}
 		/* show everything and popup */
 		return 1;
 	}
 	return 0;
 }
 
-void pl3_file_browser_cat_key_press(GdkEventKey *event)
+static void pl3_file_browser_cat_key_press(GtkWidget *tree, GdkEventKey *event, int selected_type)
 {
+	if(selected_type != file_browser_plug.id) return; 
 	if(event->state&GDK_CONTROL_MASK && event->keyval == GDK_Insert)
 	{
 		pl3_file_browser_replace_folder();
@@ -520,7 +601,7 @@ void pl3_file_browser_cat_key_press(GdkEventKey *event)
 
 }
 
-int pl3_file_browser_playlist_key_press(GtkWidget *tree, GdkEventKey *event)
+static int pl3_file_browser_playlist_key_press(GtkWidget *tree, GdkEventKey *event)
 {
 	if(event->state&GDK_CONTROL_MASK && event->keyval == GDK_Insert)
 	{
@@ -534,20 +615,32 @@ int pl3_file_browser_playlist_key_press(GtkWidget *tree, GdkEventKey *event)
 	{
 		pl3_file_browser_add_selected();
 	}
-	else if(event->keyval == GDK_i)
+	else if(event->keyval == GDK_i && event->state&GDK_MOD1_MASK)
 	{
 		pl3_file_browser_show_info();
 	}
+	else if((event->state&(GDK_CONTROL_MASK|GDK_MOD1_MASK)) == 0 && 
+			((event->keyval >= GDK_space && event->keyval <= GDK_z)))
+	{
+		char data[2];
+		data[0] = (char)gdk_keyval_to_unicode(event->keyval);
+		data[1] = '\0';
+		treesearch_start(TREESEARCH(pl3_fb_tree_search));
+		gtk_entry_set_text(GTK_ENTRY(TREESEARCH(pl3_fb_tree_search)->entry),data); 
+		gtk_editable_set_position(GTK_EDITABLE(TREESEARCH(pl3_fb_tree_search)->entry),1);
+		return TRUE;
+	}
 	else
 	{
 		return pl3_window_key_press_event(tree,event);
 	}
+
 	return TRUE;
 }
 
 
 
-void pl3_file_browser_cat_sel_changed(GtkTreeView *tree,GtkTreeIter *iter)
+static void pl3_file_browser_cat_sel_changed(GtkWidget *tree,GtkTreeIter *iter)
 {
 	long unsigned time= 0;
 	gchar *string;
@@ -555,26 +648,27 @@ void pl3_file_browser_cat_sel_changed(GtkTreeView *tree,GtkTreeIter *iter)
 	time = pl3_file_browser_view_folder(iter);
 	string = format_time(time);
 	gtk_statusbar_push(GTK_STATUSBAR(glade_xml_get_widget(pl3_xml, "statusbar2")),0, string);
-	g_free(string);
+	q_free(string);
 }
 
-void pl3_file_browser_selected()
+static void pl3_file_browser_selected(GtkWidget *container)
 {
 	if(pl3_fb_tree == NULL)
 	{
 		pl3_file_browser_init();
 	}
 
-	gtk_container_add(GTK_CONTAINER(glade_xml_get_widget(pl3_xml, "browser_container")), pl3_fb_vbox);
+	gtk_container_add(GTK_CONTAINER(container), pl3_fb_vbox);
+	gtk_widget_grab_focus(pl3_fb_tree);
 	gtk_widget_show(pl3_fb_vbox);
 }
-void pl3_file_browser_unselected()
+static void pl3_file_browser_unselected(GtkWidget *container)
 {
-	gtk_container_remove(GTK_CONTAINER(glade_xml_get_widget(pl3_xml, "browser_container")), pl3_fb_vbox);
+	gtk_container_remove(GTK_CONTAINER(container), pl3_fb_vbox);
 }
 
 
-void pl3_file_browser_show_info()
+static void pl3_file_browser_show_info()
 {
 	GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(pl3_fb_tree));
 	GtkTreeSelection *selection =gtk_tree_view_get_selection (GTK_TREE_VIEW(pl3_fb_tree));
@@ -593,19 +687,15 @@ void pl3_file_browser_show_info()
 			GtkTreeIter iter;
 			char *path;
 			int type;
-			MpdData *data;
 			gtk_tree_model_get_iter (model, &iter, (GtkTreePath *) list->data);
 			gtk_tree_model_get (GTK_TREE_MODEL(pl3_fb_store), &iter,PL3_FB_TYPE,&type, PL3_FB_PATH, &path, -1);
 			if(type == PL3_ENTRY_SONG)
 			{
-				data = mpd_database_find_adv(connection,TRUE,MPD_TAG_ITEM_FILENAME,path,-1);
-				while(data != NULL)
-				{
-					call_id3_window_song(mpd_songDup(data->song));
-					data = mpd_data_get_next(data);
-				}
+				mpd_Song *song = mpd_database_get_fileinfo(connection, path);
+				if(song)
+					call_id3_window_song(song); 
 			}
-			g_free(path);
+			q_free(path);
 		}
 		while ((list = g_list_previous (list)) && mpd_check_connected(connection));
 		/* free list */
@@ -615,22 +705,24 @@ void pl3_file_browser_show_info()
 }
 
 
-void pl3_file_browser_row_activated(GtkTreeView *tree, GtkTreePath *tp)
+static void pl3_file_browser_row_activated(GtkTreeView *tree, GtkTreePath *tp)
 {
 	GtkTreeIter iter;
-	gchar *song_id;
+	gchar *song_path;
 	gint r_type;
-	gtk_tree_model_get_iter(gtk_tree_view_get_model(tree), &iter, tp);
-	gtk_tree_model_get(gtk_tree_view_get_model(tree), &iter, PL3_SONG_ID,&song_id, PL3_SONG_POS, &r_type, -1);
-	if(song_id == NULL)
-	{
+	int playlist_length = mpd_playlist_get_playlist_length(connection);
 
+	gtk_tree_model_get_iter(gtk_tree_view_get_model(tree), &iter, tp);
+	gtk_tree_model_get(gtk_tree_view_get_model(tree), &iter, PL3_FB_PATH,&song_path, PL3_FB_TYPE, &r_type, -1);
+	if(song_path == NULL)
+	{
 		return;
 	}
 	if(r_type&PL3_ENTRY_PLAYLIST)
 	{
-		pl3_push_statusbar_message("Loaded playlist");
-		mpd_playlist_queue_load(connection, song_id);
+		pl3_push_statusbar_message(_("Loaded playlist"));
+		mpd_playlist_queue_load(connection, song_path);
+		mpd_playlist_queue_commit(connection);
 	}
 	else if (r_type&PL3_ENTRY_DIRECTORY)
 	{
@@ -648,15 +740,17 @@ void pl3_file_browser_row_activated(GtkTreeView *tree, GtkTreePath *tp)
 			{
 				do{
 					char *name = NULL;
-					gtk_tree_model_get(model, &citer, 2, &name, -1);
-					if(strcmp(name, song_id) == 0)
+					char *type= NULL;
+					gtk_tree_model_get(model, &citer, 2, &name, 3, &type,-1);
+					if(strcmp(name, song_path) == 0 && strcmp(type, "gtk-open") == 0)
 					{
 						gtk_tree_selection_select_iter(selec,&citer);
 						path = gtk_tree_model_get_path(model, &citer);
 						gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(glade_xml_get_widget (pl3_xml, "cat_tree")), path,NULL,TRUE,0.5,0);
 						gtk_tree_path_free(path);
 					}
-					g_free(name);
+					q_free(name);
+					q_free(type);
 				}while(gtk_tree_model_iter_next(model, &citer));
 			}
 
@@ -683,17 +777,28 @@ void pl3_file_browser_row_activated(GtkTreeView *tree, GtkTreePath *tp)
 	}
 	else
 	{
-		pl3_push_statusbar_message("Added a song");
-		mpd_playlist_queue_add(connection, song_id);
+		pl3_push_statusbar_message(_("Added a song"));
+		if(mpd_server_check_command_allowed(connection, "addid") == MPD_SERVER_COMMAND_ALLOWED){
+			int songid = mpd_playlist_add_get_id(connection, song_path);
+			if(songid >= 0) {
+				mpd_player_play_id(connection, songid);
+			}
+		} else{
+			mpd_playlist_add(connection, song_path);
+			if(playlist_length == 0)
+			{
+				mpd_player_play(connection);
+			}
+		}
 	}
-	mpd_playlist_queue_commit(connection);
-	g_free(song_id);
+
+	q_free(song_path);
 }
 
 
 
 
-void pl3_file_browser_button_release_event(GtkWidget *but, GdkEventButton *event)
+static void pl3_file_browser_button_release_event(GtkWidget *but, GdkEventButton *event)
 {
 
 	int has_item = 0;
@@ -739,7 +844,7 @@ void pl3_file_browser_button_release_event(GtkWidget *but, GdkEventButton *event
 			}
 			else if(row_type == PL3_ENTRY_DIRECTORY)
 			{
-				item = gtk_image_menu_item_new_with_label("Update");
+				item = gtk_image_menu_item_new_with_label(_("Update"));
 				gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
 						gtk_image_new_from_stock(GTK_STOCK_REFRESH, GTK_ICON_SIZE_MENU));
 				gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
@@ -749,11 +854,11 @@ void pl3_file_browser_button_release_event(GtkWidget *but, GdkEventButton *event
 			}
 			g_list_foreach (list,(GFunc) gtk_tree_path_free, NULL);
 			g_list_free (list);
-			g_free(path);
+			q_free(path);
 			if(row_type != PL3_ENTRY_DIR_UP)
 			{
 				/* replace the replace widget */
-				item = gtk_image_menu_item_new_with_label("Replace");
+				item = gtk_image_menu_item_new_with_label(_("Replace"));
 				gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
 						gtk_image_new_from_stock(GTK_STOCK_REDO, GTK_ICON_SIZE_MENU));
 				gtk_menu_shell_prepend(GTK_MENU_SHELL(menu), item);
@@ -773,7 +878,7 @@ void pl3_file_browser_button_release_event(GtkWidget *but, GdkEventButton *event
 	{
 
 		/* replace the replace widget */
-		item = gtk_image_menu_item_new_with_label("Replace");
+		item = gtk_image_menu_item_new_with_label(_("Replace"));
 		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
 				gtk_image_new_from_stock(GTK_STOCK_REDO, GTK_ICON_SIZE_MENU));
 		gtk_menu_shell_prepend(GTK_MENU_SHELL(menu), item);
@@ -795,21 +900,23 @@ void pl3_file_browser_button_release_event(GtkWidget *but, GdkEventButton *event
 	}
 	return;
 }
-void pl3_file_browser_replace_selected()
+static void pl3_file_browser_replace_selected()
 {
 	mpd_playlist_clear(connection);
 	pl3_file_browser_add_selected();
 	mpd_player_play(connection);
 
 }
-void pl3_file_browser_add_selected()
+static void pl3_file_browser_add_selected()
 {
 	GtkTreeIter iter;
 	GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(pl3_fb_tree));
 	GtkTreeModel *model = GTK_TREE_MODEL (pl3_fb_store);
 	GList *rows = gtk_tree_selection_get_selected_rows (selection, &model);
 	int songs=0;
-	gchar *message;
+	int dirs = 0;
+	int pl = 0;
+	/*gchar *message;*/
 	if(rows != NULL)
 	{
 		gchar *name;
@@ -825,29 +932,90 @@ void pl3_file_browser_add_selected()
 			{
 				/* add them to the add list */
 				mpd_playlist_queue_add(connection, name);
+				if(type&PL3_ENTRY_DIRECTORY) dirs++;
+				if(type&PL3_ENTRY_SONG) songs++;
 			}
 			else if (type == PL3_ENTRY_PLAYLIST)
 			{
 				mpd_playlist_queue_load(connection, name);
+				pl++;
 			}
-			songs++;
-			g_free(name);
+			q_free(name);
 		}while((node = g_list_next(node)) != NULL);
 	}
 	/* if there are items in the add list add them to the playlist */
 	mpd_playlist_queue_commit(connection);
-	if(songs != 0)
+	if((songs+dirs+pl) != 0)
 	{
-		message = g_strdup_printf("Added %i song%s", songs, (songs != 1)? "s":"");
-		pl3_push_statusbar_message(message);
-		g_free(message);
+		GString *string= g_string_new(_("Added"));
+		if(songs)
+			g_string_append_printf(string, " %i %s%c", songs, (songs >1)? _("songs"):_("songs"), (dirs+pl >0)?',':' ');
+		if(dirs)
+			g_string_append_printf(string, " %i %s%c", dirs, (dirs>1)?_("directories"):_("directory"), (pl>0)?',':' ');
+		if(pl)
+			g_string_append_printf(string, " %i %s", pl, (pl>1)?_("playlists"):_("playlist"));
+		pl3_push_statusbar_message(string->str);
+		g_string_free(string, TRUE);
 	}
 
 	g_list_foreach (rows, (GFunc) gtk_tree_path_free, NULL);
 	g_list_free (rows);
 }
 
-void pl3_file_browser_delete_playlist(GtkMenuItem *bt)
+static void pl3_file_browser_delete_playlist_from_right(GtkMenuItem *bt)
+{
+	GtkTreeView *tree = playlist3_get_category_tree_view();
+	GtkTreeModel *model = (GtkTreeModel *) playlist3_get_category_tree_store();
+	GtkTreeSelection *selection  = gtk_tree_view_get_selection(tree);
+	GtkTreeIter iter;
+	char *path= NULL;
+	/* create a warning message dialog */
+	GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW
+			(glade_xml_get_widget
+			 (pl3_xml, "pl3_win")),
+			GTK_DIALOG_MODAL,
+			GTK_MESSAGE_WARNING,
+			GTK_BUTTONS_NONE,
+			_("Are you sure you want to clear the selected playlist?"));
+
+	gtk_dialog_add_buttons (GTK_DIALOG (dialog), GTK_STOCK_NO,
+			GTK_RESPONSE_CANCEL, GTK_STOCK_YES,
+			GTK_RESPONSE_OK, NULL);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+			GTK_RESPONSE_CANCEL);
+
+
+	if(gtk_tree_selection_get_selected(selection, &model, &iter))
+	{
+		char *icon = NULL;
+		gtk_tree_model_get(model, &iter,3, &icon,2, &path, -1);
+		if(path && strcmp("media-playlist", icon)) {
+			if(path)
+				q_free(path);
+			path = NULL;
+		}
+		q_free(icon);
+	}
+
+	printf("path: %s\n", path);
+
+	if(path == NULL){
+		gtk_widget_destroy(dialog);
+		return;
+	}	
+
+	switch (gtk_dialog_run (GTK_DIALOG (dialog)))
+	{
+		case GTK_RESPONSE_OK:
+			mpd_database_delete_playlist(connection, path);
+			/*pl3_cat_sel_changed();*/
+			pl3_file_browser_reupdate();
+	}
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+	q_free(path);
+}
+
+static void pl3_file_browser_delete_playlist(GtkMenuItem *bt)
 {
 	char *path= NULL;
 	GtkTreeSelection *sel = NULL;
@@ -883,7 +1051,7 @@ void pl3_file_browser_delete_playlist(GtkMenuItem *bt)
 		}
 	}
 
-	
+
 	if(path == NULL){
 		gtk_widget_destroy(dialog);
 		return;
@@ -893,14 +1061,15 @@ void pl3_file_browser_delete_playlist(GtkMenuItem *bt)
 	{
 		case GTK_RESPONSE_OK:
 			mpd_database_delete_playlist(connection, path);
-			pl3_cat_sel_changed();
+			pl3_file_browser_reupdate();
+/*			pl3_cat_sel_changed();*/
 
 	}
 	gtk_widget_destroy (GTK_WIDGET (dialog));
-	g_free(path);
+	q_free(path);
 }
 
-void pl3_file_browser_disconnect()
+static void pl3_file_browser_disconnect()
 {
 
 	if(pl3_fb_tree_ref) {
@@ -921,3 +1090,60 @@ void pl3_file_browser_disconnect()
 	}
 	if(pl3_fb_store)gtk_list_store_clear(pl3_fb_store);
 }
+
+
+static void pl3_file_browser_activate(void)
+{
+	GtkTreeSelection *selec = gtk_tree_view_get_selection((GtkTreeView *)
+			playlist3_get_category_tree_view());
+
+	/**
+	 * Fix this to be nnot static
+	 */	
+	GtkTreePath *path = gtk_tree_row_reference_get_path(pl3_fb_tree_ref); 
+	if(path)
+	{
+		gtk_tree_selection_select_path(selec, path);
+		gtk_tree_path_free(path);
+	}
+}
+
+
+static int pl3_file_browser_add_go_menu(GtkWidget *menu)
+{
+	GtkWidget *item = NULL;
+
+	item = gtk_image_menu_item_new_with_label(_("File Browser"));
+	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), 
+			gtk_image_new_from_stock("gtk-open", GTK_ICON_SIZE_MENU));
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+	g_signal_connect(G_OBJECT(item), "activate", 
+			G_CALLBACK(pl3_file_browser_activate), NULL);
+	return 1;
+}
+
+static void pl3_file_browser_connection_changed(MpdObj *mi, int connect, gpointer data)
+{
+	if(!connect)
+	{
+		pl3_file_browser_disconnect();
+	}
+}
+
+static void pl3_file_browser_status_changed(MpdObj *mi,ChangedStatusType what, void *data)
+{
+	if(what&MPD_CST_DATABASE)
+	{
+		pl3_file_browser_reupdate();
+	}
+}	
+static int pl3_file_browser_key_press_event(GtkWidget *mw, GdkEventKey *event, int type)
+{
+	if (event->keyval == GDK_F3)
+	{
+		pl3_file_browser_activate();
+		return TRUE;
+	}                                           	
+
+	return FALSE;
+}
