@@ -1,6 +1,9 @@
 #include "config1.h"
+
 #include <gtk/gtk.h>
 #include <libmpd/libmpd.h>
+#include "metadata.h"
+
 #ifndef __GMPC_PLUGIN_H__
 #define __GMPC_PLUGIN_H__
 
@@ -11,11 +14,12 @@ extern config_obj *config;
 
 /* Plugin Type's */
 typedef enum {
-	GMPC_PLUGIN_DUMMY	= 1,
+	GMPC_PLUGIN_DUMMY		= 1,
 	GMPC_PLUGIN_PL_BROWSER	= 2,
-	GMPC_PLUGIN_NO_GUI	= 4,
-	GMPC_INTERNALL		= 8,
-	GMPC_PLUGIN_COVER_ART	= 16
+	GMPC_PLUGIN_NO_GUI		= 4,
+	GMPC_INTERNALL			= 8,
+	GMPC_DEPRECATED			= 16,
+	GMPC_PLUGIN_META_DATA	= 32
 } PluginType;
 
 /* usefull defines */
@@ -29,15 +33,15 @@ typedef enum {
 #define PL3_ENTRY_SONG 1
 
 /* the gtk_tree_store row's */
-enum
+typedef enum
 {
-	PL3_CAT_TYPE,
-	PL3_CAT_TITLE,
-	PL3_CAT_INT_ID,
-	PL3_CAT_ICON_ID,
+	PL3_CAT_TYPE, /** Plugin id */
+	PL3_CAT_TITLE, /** title that is showed */
+	PL3_CAT_INT_ID, /* id */
+	PL3_CAT_ICON_ID, /* icon id */
 	PL3_CAT_PROC, /* for the lazy tree, if the dir is allready processed */
-	PL3_CAT_ICON_SIZE,
-	PL3_CAT_BROWSE_FORMAT,
+	PL3_CAT_ICON_SIZE, /** icon size */
+	PL3_CAT_BROWSE_FORMAT, /** string, added for tag browser */
 	PL3_CAT_NROWS
 } pl3_cat_store;
 
@@ -56,24 +60,40 @@ typedef struct {
 /** gmpcPlBrowserPlugin, functions needed for intergration with the playlist browser
  */
 typedef struct {
+	/**
+	 * Adding to the left side tree */
 	void (*add)(GtkWidget *cat_tree);
+	/**
+	 * If selected,  you need to fill the right screen */
 	void (*selected)(GtkWidget *container);
+	/** 
+	 * if unselected, you need to remove youself from the container
+	 */
 	void (*unselected)(GtkWidget *container);
+	/**
+	 * if selection changed, but still the same plugin 
+	 */
 	void (*cat_selection_changed)(GtkWidget *tree, GtkTreeIter *iter);
-	void (*fill_tree)(GtkTreeView *tree,GtkTreeIter *iter);
-	int (*cat_right_mouse_menu)(GtkWidget *menu, int type, GtkWidget *tree,GdkEventButton *event);
+	/**
+	 * if the user expands the tree.
+	 */
+	void (*cat_row_expanded)(GtkWidget *tree,GtkTreeIter *iter);
+	int  (*cat_right_mouse_menu)(GtkWidget *menu, int type, GtkWidget *tree,GdkEventButton *event);
 	void (*cat_key_press)(GtkWidget *tree, GdkEventKey *event, int selected_type);
-	/* not yet implemented */
+	/****** GO MENU ********/
+	int  (*add_go_menu)(GtkWidget *menu); 
+	/****** Key presses (in the whole window) **/
+	int  (*key_press_event)(GtkWidget *mw, GdkEventKey *event, int type);
 } gmpcPlBrowserPlugin;
 
 typedef struct {
-	int (*fetch_image_path)(mpd_Song *song, gchar **path);
-	int (*fetch_image)(mpd_Song *song, GSourceFunc function);
-	int (*get_priority)(); /* the lower the better */
-} gmpcCoverArtPlugin;
+	int (*get_priority)();
+	int (*get_image)(mpd_Song *song, MetaDataType type, char **path);
+} gmpcMetaDataPlugin;
 
 /* Unique number */
-#define PLUGIN_API_VERSION 11
+/* 16 == 0.14 release */
+#define PLUGIN_API_VERSION 16
 
 /* sturcture */
 typedef struct {
@@ -86,6 +106,7 @@ typedef struct {
 	char			*path;	/* Do not fill in, done by gmpc */
 	/* function gets called on startup */
 	void			(*init)(void);
+        void                    (*destroy)(void);
 	/* Browser Plugins */
 	gmpcPlBrowserPlugin	*browser;
 	/* plugin with one signal for changes on Mpd */
@@ -94,15 +115,17 @@ typedef struct {
 	ConnectionChangedCallback mpd_connection_changed;
 	/* structure to let the plugin intergrate it's preferences */
 	gmpcPrefPlugin		*pref;
-	/* Cover art plugin */
-	gmpcCoverArtPlugin	*coverart;
-	
+	/** Meta data */
+	gmpcMetaDataPlugin *metadata;
+
+	/** Plugin control functions
+	 */
+	int	 	(*get_enabled)();
+	void	(*set_enabled)(int enable);
 } gmpcPlugin;
 
-
-/* Playlist 3 helper functions */
-gboolean playlist3_get_active();
-GtkTreeStore *playlist3_get_category_tree_store();
-GtkTreeView *playlist3_get_category_tree_view();
-
+/** plugin functions */
+gmpcPlugin * plugin_get_from_id(int id);
+GtkTreeStore *playlist3_get_category_tree_store(void);
+GtkTreeView *playlist3_get_category_tree_view(void);
 #endif
