@@ -13,46 +13,27 @@
 
 #define PLUGIN_STATS -200
 
-void plugin_stats_construct(GtkWidget *container);
-void plugin_stats_destroy(GtkWidget *container);
+static void plugin_stats_construct(GtkWidget *);
+static void plugin_stats_destroy(GtkWidget *);
 GladeXML *plugin_stat_xml = NULL;
 /* About "plugin" */
-void about_pref_construct(GtkWidget *container);
-void about_pref_destroy(GtkWidget *container);
-GladeXML *about_pref_xml = NULL;
-gmpcPrefPlugin about_gpp = {
-	about_pref_construct,
-	about_pref_destroy
-};
 
-gmpcPlugin about_plug = {
-	"About",
-	{1,1,1},
-	GMPC_INTERNALL,
-	0,
-	NULL, /* initialize */
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	&about_gpp
-};
 /* End About */
 GtkListStore *plugin_store = NULL;
 GladeXML *xml_preferences_window = NULL;
-gboolean running = 0, connected = 0;
+gboolean running = 0;
 
-void update_auth_settings();
+/* Glade Prototypes, without glade these would be static */
 void preferences_window_connect(GtkWidget *but);
 void preferences_window_disconnect(GtkWidget *but);
 void auth_enable_toggled(GtkToggleButton *but);
 void entry_auth_changed(GtkEntry *entry);
-void set_display_settings();
-void update_display_settings();
+void create_preferences_window(void);
+void preferences_window_destroy(void);
 
 int plugin_last;
 
-void pref_plugin_changed()
+static void pref_plugin_changed()
 {
 	GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW (glade_xml_get_widget(xml_preferences_window, "plugin_tree")));
 	GtkTreeModel *model = GTK_TREE_MODEL(plugin_store);
@@ -76,7 +57,7 @@ void pref_plugin_changed()
 			if(plugins[id]->pref->construct)
 			{
 				char *buf = NULL;
-				if(plugins[id]->plugin_type != GMPC_INTERNALL)
+				if(plugins[id]->plugin_type != GMPC_INTERNALL && !(plugins[id]->id&PLUGIN_ID_INTERNALL))
 				{
 					buf = g_strdup_printf("<span size=\"xx-large\"><b>%s</b></span>\n<i>Plugin version: %i.%i.%i</i>", 
 							plugins[id]->name,
@@ -91,7 +72,7 @@ void pref_plugin_changed()
 				plugins[id]->pref->construct(glade_xml_get_widget(xml_preferences_window, "plugin_container"));
 				plugin_last = id;
 				gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml_preferences_window, "plugin_label")),buf);
-				g_free(buf);
+				q_free(buf);
 				return;
 			}
 		}
@@ -137,7 +118,7 @@ void create_preferences_window()
 	plugin_last = -1;
 	string = gmpc_get_full_glade_path("gmpc.glade");
 	xml_preferences_window = glade_xml_new(string, "preferences_window", NULL);
-	g_free(string);
+	q_free(string);
 	/* check for errors and axit when there is no gui file */
 	if(xml_preferences_window == NULL)  g_error("Couldnt initialize GUI. Please check installation\n");
 
@@ -148,7 +129,7 @@ void create_preferences_window()
 	gtk_widget_show_all(GTK_WIDGET(dialog));
 	running = 1;
 
-	plugin_store = gtk_list_store_new(2, GTK_TYPE_INT, GTK_TYPE_STRING);
+	plugin_store = gtk_list_store_new(2, G_TYPE_INT, G_TYPE_STRING);
 	column = gtk_tree_view_column_new ();
 	gtk_tree_view_column_set_title(column, _("Plugins:"));
 	renderer = gtk_cell_renderer_text_new ();
@@ -187,7 +168,7 @@ void create_preferences_window()
 		}
 	}
 	/* plugins */
-	if(plugs)
+/*	if(plugs) */
 	{
 		GtkTreeIter iter;
 		gtk_list_store_append(GTK_LIST_STORE(plugin_store), &iter);
@@ -206,16 +187,12 @@ void create_preferences_window()
 	}
 
 	label = glade_xml_get_widget(xml_preferences_window, "plugin_label_box");
-	gtk_widget_modify_bg(label, GTK_STATE_NORMAL, &dialog->style->bg[GTK_STATE_SELECTED]);
-	label = glade_xml_get_widget(xml_preferences_window, "plugin_label");
+	gtk_widget_modify_bg(label, GTK_STATE_NORMAL, &dialog->style->light[GTK_STATE_SELECTED]);
+/*	label = glade_xml_get_widget(xml_preferences_window, "plugin_label");
 	gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &dialog->style->fg[GTK_STATE_SELECTED]);
-	glade_xml_signal_autoconnect(xml_preferences_window);	
+*/	glade_xml_signal_autoconnect(xml_preferences_window);	
 
 }
-void set_display_default_sd()
-{
-	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml_preferences_window, "en_sd")), DEFAULT_PLAYER_MARKUP);
-}
 
 /* destory the preferences window */
 void preferences_window_destroy()
@@ -233,80 +210,132 @@ void preferences_window_destroy()
 	}
 	gtk_widget_destroy(dialog);
 	g_object_unref(xml_preferences_window);
+	g_object_unref(plugin_store);
+	plugin_store = NULL;
 	xml_preferences_window = NULL;
 	running = 0;
 }
 
-void about_pref_destroy(GtkWidget *container)
+static void pref_plugin_enabled(GtkCellRendererToggle *rend, gchar *path, GtkListStore *store)
 {
-	if(about_pref_xml)
+	GtkTreeIter iter;
+	if(gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(store), &iter, path))
 	{
-		GtkWidget *vbox = glade_xml_get_widget(about_pref_xml, "about-vbox");
-		gtk_container_remove(GTK_CONTAINER(container),vbox);
-		g_object_unref(about_pref_xml);
-		about_pref_xml = NULL;
+		int toggled;
+		gmpcPlugin *plug = NULL;
+		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 0, &toggled, 3, &plug, -1);
+		if(plug->set_enabled != NULL)
+		{
+			plug->set_enabled(!toggled);
+			gtk_list_store_set(store, &iter, 0,plug->get_enabled(),-1);
+		}
+
+
 	}
+
 }
-void about_pref_construct(GtkWidget *container)
+static void plugin_stats_construct(GtkWidget *container)
 {
 	gchar *path = gmpc_get_full_glade_path("gmpc.glade");
-	about_pref_xml = glade_xml_new(path, "about-vbox",NULL);
-
-	if(about_pref_xml)
-	{
-		GtkWidget *vbox = glade_xml_get_widget(about_pref_xml, "about-vbox");
-		gtk_container_add(GTK_CONTAINER(container),vbox);
-	}
-}
-
-
-void plugin_stats_construct(GtkWidget *container)
-{
-	gchar *path = gmpc_get_full_glade_path("gmpc.glade");
-	plugin_stat_xml = glade_xml_new(path, "plugin_stat_tb",NULL);
-	g_free(path);
+	plugin_stat_xml = glade_xml_new(path, "plugin_stats_vbox",NULL);
+	q_free(path);
 	if(plugin_stat_xml)
 	{
-		int plug_brow = 0, plug_misc =0,i=0;
-		int cover_prov = 0;
-		GtkWidget *vbox = glade_xml_get_widget(plugin_stat_xml, "plugin_stat_tb");
+		GtkWidget *tree = glade_xml_get_widget(plugin_stat_xml, "plugin_stats_tree");
+		GtkListStore *store = NULL;
+		GtkTreeIter iter;
+		GtkCellRenderer *renderer = NULL;
+		int i=0;
+		GtkWidget *vbox = glade_xml_get_widget(plugin_stat_xml, "plugin_stats_vbox");
+
+
+		/**
+		 * new 
+		 */
+		store = gtk_list_store_new(4,G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING,G_TYPE_POINTER);
+		gtk_tree_view_set_model(GTK_TREE_VIEW(tree),GTK_TREE_MODEL(store));
+		renderer = gtk_cell_renderer_toggle_new();
+		g_object_set_data(G_OBJECT(renderer), "editable", GINT_TO_POINTER(1));
+		gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW(tree), -1,_("Enabled"), renderer, "active", 0,NULL);	
+		g_signal_connect(G_OBJECT(renderer), "toggled", G_CALLBACK(pref_plugin_enabled), store);
+		renderer = gtk_cell_renderer_text_new();
+		gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW(tree), -1,_("Name"), renderer, "text", 1,NULL);
+		renderer = gtk_cell_renderer_text_new();
+		gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW(tree), -1,_("Function"), renderer, "text", 2,NULL);
 		for(i=0;i<num_plugins;i++)
 		{
 			if(plugins[i]->id&PLUGIN_ID_MARK)
 			{
-				if(plugins[i]->plugin_type == GMPC_PLUGIN_PL_BROWSER) plug_brow++;
-				else if (plugins[i]->plugin_type == GMPC_PLUGIN_NO_GUI) plug_misc++;
-				else if (plugins[i]->plugin_type == GMPC_PLUGIN_COVER_ART)cover_prov++; 
+				gtk_list_store_append(store, &iter);
+				gtk_list_store_set(store, &iter, 0,TRUE,1, plugins[i]->name,3,(plugins[i]), -1);
+				if(plugins[i]->get_enabled != NULL)
+				{
+					gtk_list_store_set(store, &iter, 0,plugins[i]->get_enabled(),-1);
+				}
+				switch(plugins[i]->plugin_type)
+				{
+					case GMPC_PLUGIN_DUMMY:
+						gtk_list_store_set(store, &iter, 2, _("Dummy"),-1);
+						break;
+					case GMPC_PLUGIN_PL_BROWSER:
+						gtk_list_store_set(store, &iter, 2, _("Browser Extention"),-1);
+						break;
+					case GMPC_PLUGIN_META_DATA:
+						gtk_list_store_set(store, &iter, 2, _("Metadata Provider"),-1);
+						break;
+
+
+					case GMPC_PLUGIN_NO_GUI:
+						gtk_list_store_set(store, &iter, 2, _("Misc."),-1);
+						break;                                                         					
+
+					default:
+						gtk_list_store_set(store, &iter, 2, _("Unkown"),-1);
+						break;
+
+				}
 			}
+
 		}
-		path = g_strdup_printf("%i", plug_brow+plug_misc+cover_prov);
-		gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(plugin_stat_xml, "num_plug_label")),path);
-		g_free(path);
-
-		path = g_strdup_printf("%i", plug_brow);
-		gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(plugin_stat_xml, "num_brow_label")),path);
-		g_free(path);
-
-		path = g_strdup_printf("%i", plug_misc);
-		gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(plugin_stat_xml, "num_misc_label")),path);
-		g_free(path);
-
-		path = g_strdup_printf("%i", cover_prov);
-		gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(plugin_stat_xml, "cover_prov_label")),path);
-		g_free(path);
-
-		
 		gtk_container_add(GTK_CONTAINER(container),vbox);
 	}
 
 }
-void plugin_stats_destroy(GtkWidget *container)
+static void plugin_stats_destroy(GtkWidget *container)
 {
 	if(plugin_stat_xml)
 	{
-		GtkWidget *vbox = glade_xml_get_widget(plugin_stat_xml, "plugin_stat_tb");
+		GtkWidget *vbox = glade_xml_get_widget(plugin_stat_xml, "plugin_stats_vbox");
+		GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(glade_xml_get_widget(plugin_stat_xml, "plugin_stats_tree")));
 		gtk_container_remove(GTK_CONTAINER(container),vbox);
+		if(model)g_object_unref(model);
 		g_object_unref(plugin_stat_xml);
 		plugin_stat_xml = NULL;
 	}
 }
+
+
+void preferences_show_pref_window(int plugin_id)
+{
+	GtkTreeIter iter;
+	if(xml_preferences_window == NULL)
+		create_preferences_window();
+	if(plugin_store)
+	{
+		if(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(plugin_store), &iter))
+		{
+			do{	
+				int pos;
+				gtk_tree_model_get(GTK_TREE_MODEL(plugin_store), &iter, 0, &pos, -1); 
+				if(pos == plugin_get_pos(plugin_id))
+				{
+					GtkTreeSelection *select = gtk_tree_view_get_selection(
+							GTK_TREE_VIEW (glade_xml_get_widget(xml_preferences_window, "plugin_tree")));
+					gtk_tree_selection_select_iter(select, &iter);
+					return;
+				}
+			}while(gtk_tree_model_iter_next(GTK_TREE_MODEL(plugin_store),  &iter));
+		}
+	}	
+
+}
